@@ -7,6 +7,8 @@
 #include "util/picture_load.hpp"
 #include <list>
 #include "nlohmann/json.hpp"
+#include "pool/ipool.hpp"
+#include "pool/pool_cached.hpp"
 
 namespace horizon {
 
@@ -30,8 +32,11 @@ BoardColors::BoardColors() : solder_mask({0, .5, 0}), substrate({.2, .15, 0})
 {
 }
 
-Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadstackProvider &vpp)
-    : uuid(uu), block(&iblock), name(j.at("name").get<std::string>()), n_inner_layers(j.value("n_inner_layers", 0))
+static const unsigned int app_version = 0;
+
+Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool, ViaPadstackProvider &vpp)
+    : uuid(uu), block(&iblock), name(j.at("name").get<std::string>()), version(app_version, j),
+      n_inner_layers(j.value("n_inner_layers", 0))
 {
     Logger::log_info("loading board " + (std::string)uuid, Logger::Domain::BOARD);
     if (j.count("stackup")) {
@@ -54,7 +59,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
         const json &o = j["holes"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            load_and_log(holes, ObjectType::BOARD_HOLE, std::forward_as_tuple(u, it.value(), block, &pool),
+            load_and_log(holes, ObjectType::BOARD_HOLE, std::forward_as_tuple(u, it.value(), block, pool),
                          Logger::Domain::BOARD);
         }
     }
@@ -196,6 +201,13 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
             load_and_log(pictures, ObjectType::PICTURE, std::forward_as_tuple(u, it.value()), Logger::Domain::BOARD);
         }
     }
+    if (j.count("decals")) {
+        const json &o = j["decals"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID(it.key());
+            load_and_log(decals, ObjectType::DECAL, std::forward_as_tuple(u, it.value(), pool), Logger::Domain::BOARD);
+        }
+    }
     if (j.count("rules")) {
         try {
             rules.load_from_json(j.at("rules"));
@@ -247,19 +259,19 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, Pool &pool, ViaPadsta
             Logger::log_warning("couldn't load p&p export settings", Logger::Domain::BOARD, e.what());
         }
     }
-    fab_output_settings.update_for_board(this);
+    fab_output_settings.update_for_board(*this);
     update_pdf_export_settings(pdf_export_settings);
     update_refs(); // fill in smashed texts
 }
 
-Board Board::new_from_file(const std::string &filename, Block &block, Pool &pool, ViaPadstackProvider &vpp)
+Board Board::new_from_file(const std::string &filename, Block &block, IPool &pool, ViaPadstackProvider &vpp)
 {
     auto j = load_json_from_file(filename);
     return Board(UUID(j.at("uuid").get<std::string>()), j, block, pool, vpp);
 }
 
 
-Board::Board(const UUID &uu, Block &bl) : uuid(uu), block(&bl)
+Board::Board(const UUID &uu, Block &bl) : uuid(uu), block(&bl), version(app_version)
 {
     rules.add_rule(RuleID::CLEARANCE_COPPER);
     rules.add_rule(RuleID::CLEARANCE_COPPER_OTHER);
@@ -290,11 +302,11 @@ Board::Board(const Board &brd, CopyMode copy_mode)
     : layers(brd.layers), uuid(brd.uuid), block(brd.block), name(brd.name), polygons(brd.polygons), holes(brd.holes),
       junctions(brd.junctions), tracks(brd.tracks), texts(brd.texts), lines(brd.lines), arcs(brd.arcs),
       planes(brd.planes), keepouts(brd.keepouts), dimensions(brd.dimensions), connection_lines(brd.connection_lines),
-      included_boards(brd.included_boards), board_panels(brd.board_panels), pictures(brd.pictures),
+      included_boards(brd.included_boards), board_panels(brd.board_panels), pictures(brd.pictures), decals(brd.decals),
       warnings(brd.warnings), rules(brd.rules), fab_output_settings(brd.fab_output_settings), airwires(brd.airwires),
       stackup(brd.stackup), colors(brd.colors), pdf_export_settings(brd.pdf_export_settings),
       step_export_settings(brd.step_export_settings), pnp_export_settings(brd.pnp_export_settings),
-      n_inner_layers(brd.n_inner_layers)
+      version(brd.version), n_inner_layers(brd.n_inner_layers)
 {
     if (copy_mode == CopyMode::DEEP) {
         packages = brd.packages;
@@ -415,7 +427,7 @@ void Board::set_n_inner_layers(unsigned int n)
         stackup.emplace(-j, -j);
     }
     map_erase_if(stackup, [this](const auto &x) { return layers.count(x.first) == 0; });
-    fab_output_settings.update_for_board(this);
+    fab_output_settings.update_for_board(*this);
     update_pdf_export_settings(pdf_export_settings);
 }
 
@@ -574,6 +586,7 @@ void Board::expand(bool careful)
     }
 
     expand_packages();
+    expand_decals();
 
     for (auto &it : junctions) {
         it.second.layer = 10000;
@@ -587,16 +600,9 @@ void Board::expand(bool careful)
             if (it_ft.is_junc()) {
                 auto ju = it_ft.junc;
                 ju->connection_count++;
-                if (ju->layer == 10000) { // none assigned
-                    ju->layer = it.second.layer;
-                }
-                else if (ju->layer == 10001) { // invalid
-                                               // nop
-                }
-                else if (ju->layer != it.second.layer) {
-                    ju->layer = 10001;
+                ju->layer.merge(it.second.layer);
+                if (ju->layer.is_multilayer())
                     ju->needs_via = true;
-                }
             }
         }
         if (it.second.from.get_position() == it.second.to.get_position()) {
@@ -608,9 +614,7 @@ void Board::expand(bool careful)
         it.second.from->connection_count++;
         it.second.to->connection_count++;
         for (const auto &ju : {it.second.from, it.second.to}) {
-            if (ju->layer == 10000) { // none assigned
-                ju->layer = it.second.layer;
-            }
+            ju->layer.merge(it.second.layer);
         }
     }
     for (const auto &it : arcs) {
@@ -618,9 +622,7 @@ void Board::expand(bool careful)
         it.second.to->connection_count++;
         it.second.center->connection_count++;
         for (const auto &ju : {it.second.from, it.second.to, it.second.center}) {
-            if (ju->layer == 10000) { // none assigned
-                ju->layer = it.second.layer;
-            }
+            ju->layer.merge(it.second.layer);
         }
     }
 
@@ -691,6 +693,29 @@ void Board::expand(bool careful)
     airwires_expand.clear();
 }
 
+void Board::expand_decals()
+{
+    for (auto &[uu, dec] : decals) {
+        dec.decal = *dec.pool_decal;
+        dec.placement.mirror = dec.flip;
+        if (dec.flip) {
+            for (auto &it2 : dec.decal.lines) {
+                flip_package_layer(it2.second.layer);
+            }
+            for (auto &it2 : dec.decal.arcs) {
+                flip_package_layer(it2.second.layer);
+            }
+            for (auto &it2 : dec.decal.texts) {
+                flip_package_layer(it2.second.layer);
+            }
+            for (auto &it2 : dec.decal.polygons) {
+                flip_package_layer(it2.second.layer);
+            }
+        }
+        dec.apply_scale();
+    }
+}
+
 void Board::expand_packages()
 {
     auto params = rules.get_parameters();
@@ -705,8 +730,8 @@ void Board::expand_packages()
     bool expanded_package = false;
     for (auto &it : packages) {
         it.second.pool_package = it.second.component->part->package;
-        it.second.model = it.second.component->part->get_model();
         if ((expand_flags & EXPAND_PACKAGES) && (packages_expand.size() == 0 || packages_expand.count(it.first))) {
+            it.second.model = it.second.component->part->get_model();
             expanded_package = true;
             if (it.second.alternate_package) {
                 std::set<std::string> pads_from_primary, pads_from_alt;
@@ -744,6 +769,10 @@ void Board::expand_packages()
                             pad.uuid = uu;
                         }
                     }
+
+                    if (it.second.package.models.size() == 1) { // alt pkg only has one model
+                        it.second.model = it.second.package.models.begin()->first;
+                    }
                 }
             }
             else {
@@ -1141,6 +1170,7 @@ std::map<const BoardPackage *, PnPRow> Board::get_PnP(const PnPExportSettings &s
 json Board::serialize() const
 {
     json j;
+    version.serialize(j);
     j["type"] = "board";
     j["uuid"] = (std::string)uuid;
     j["block"] = (std::string)block->uuid;
@@ -1231,6 +1261,12 @@ json Board::serialize() const
             j["pictures"][(std::string)it.first] = it.second.serialize();
         }
     }
+    if (decals.size()) {
+        j["decals"] = json::object();
+        for (const auto &it : decals) {
+            j["decals"][(std::string)it.first] = it.second.serialize();
+        }
+    }
     return j;
 }
 
