@@ -10,12 +10,21 @@
 #include <sstream>
 
 namespace horizon {
-RulesCheckResult BoardRules::check_track_width(const Board *brd) const
+
+static std::string get_net_name(const Net *net)
+{
+    if (!net || net->name.size() == 0)
+        return "";
+    else
+        return "(" + net->name + ")";
+}
+
+RulesCheckResult BoardRules::check_track_width(const Board &brd) const
 {
     RulesCheckResult r;
     r.level = RulesCheckErrorLevel::PASS;
     auto rules = get_rules_sorted<RuleTrackWidth>(RuleID::TRACK_WIDTH);
-    for (const auto &it : brd->tracks) {
+    for (const auto &it : brd.tracks) {
         auto width = it.second.width;
         Net *net = it.second.net;
         auto layer = it.second.layer;
@@ -64,12 +73,12 @@ static RulesCheckError *check_hole(RulesCheckResult &r, uint64_t dia, const Rule
     return nullptr;
 }
 
-RulesCheckResult BoardRules::check_hole_size(const Board *brd) const
+RulesCheckResult BoardRules::check_hole_size(const Board &brd) const
 {
     RulesCheckResult r;
     r.level = RulesCheckErrorLevel::PASS;
     auto rules = get_rules_sorted<RuleHoleSize>(RuleID::HOLE_SIZE);
-    for (const auto &it : brd->holes) {
+    for (const auto &it : brd.holes) {
         Net *net = it.second.net;
         for (const auto &it_hole : it.second.padstack.holes) {
             auto dia = it_hole.second.diameter;
@@ -84,7 +93,7 @@ RulesCheckResult BoardRules::check_hole_size(const Board *brd) const
         }
     }
 
-    for (const auto &it : brd->vias) {
+    for (const auto &it : brd.vias) {
         Net *net = it.second.junction->net;
         for (const auto &it_hole : it.second.padstack.holes) {
             auto dia = it_hole.second.diameter;
@@ -99,7 +108,7 @@ RulesCheckResult BoardRules::check_hole_size(const Board *brd) const
         }
     }
 
-    for (const auto &it : brd->packages) {
+    for (const auto &it : brd.packages) {
         for (const auto &it_pad : it.second.package.pads) {
             Net *net = it_pad.second.net;
             for (const auto &it_hole : it_pad.second.padstack.holes) {
@@ -144,7 +153,7 @@ static std::pair<ClipperLib::IntPoint, ClipperLib::IntPoint> get_patch_bb(const
 static std::deque<RulesCheckError>
 clearance_cu_worker(std::set<std::pair<CanvasPatch::PatchKey, CanvasPatch::PatchKey>> &patch_pairs,
                     std::mutex &patch_pair_mutex, const std::map<CanvasPatch::PatchKey, ClipperLib::Paths> &patches,
-                    const Board *brd, const BoardRules *rules, int n_patch_pairs, check_status_cb_t status_cb)
+                    const Board &brd, const BoardRules &rules, int n_patch_pairs, check_status_cb_t status_cb)
 {
     std::deque<RulesCheckError> r_errors;
     while (true) {
@@ -167,12 +176,12 @@ clearance_cu_worker(std::set<std::pair<CanvasPatch::PatchKey, CanvasPatch::Patch
         auto p1 = patch_pair.first;
         auto p2 = patch_pair.second;
 
-        Net *net1 = p1.net ? &brd->block->nets.at(p1.net) : nullptr;
-        Net *net2 = p2.net ? &brd->block->nets.at(p2.net) : nullptr;
+        Net *net1 = p1.net ? &brd.block->nets.at(p1.net) : nullptr;
+        Net *net2 = p2.net ? &brd.block->nets.at(p2.net) : nullptr;
 
         // figure out the clearance between this patch pair
         uint64_t clearance = 0;
-        auto rule_clearance = rules->get_clearance_copper(net1, net2, p1.layer);
+        auto rule_clearance = rules.get_clearance_copper(net1, net2, p1.layer);
         if (rule_clearance) {
             clearance = rule_clearance->get_clearance(p1.type, p2.type);
         }
@@ -224,7 +233,7 @@ clearance_cu_worker(std::set<std::pair<CanvasPatch::PatchKey, CanvasPatch::Patch
     return r_errors;
 }
 
-RulesCheckResult BoardRules::check_clearance_copper(const Board *brd, RulesCheckCache &cache,
+RulesCheckResult BoardRules::check_clearance_copper(const Board &brd, RulesCheckCache &cache,
                                                     check_status_cb_t status_cb) const
 {
     RulesCheckResult r;
@@ -234,7 +243,7 @@ RulesCheckResult BoardRules::check_clearance_copper(const Board *brd, RulesCheck
     std::set<int> layers;
     const auto &patches = c->get_canvas()->get_patches();
     for (const auto &it : patches) { // collect copper layers
-        if (brd->get_layers().count(it.first.layer) && brd->get_layers().at(it.first.layer).copper) {
+        if (brd.get_layers().count(it.first.layer) && brd.get_layers().at(it.first.layer).copper) {
             layers.emplace(it.first.layer);
         }
     }
@@ -258,6 +267,17 @@ RulesCheckResult BoardRules::check_clearance_copper(const Board *brd, RulesCheck
                         patch_pairs.emplace(k);
                     }
                 }
+                else if (layer == it.first.layer && it.first.net != it_other.first.net
+                         && it.first.type != PatchType::OTHER && it.first.type != PatchType::TEXT
+                         && it_other.first.type == PatchType::HOLE_PTH) {
+                    // ignore layer of plated holes, as it's expected to be different from current layer
+                    std::pair<CanvasPatch::PatchKey, CanvasPatch::PatchKey> k = {it.first, it_other.first};
+                    auto k2 = k;
+                    std::swap(k2.first, k2.second);
+                    if (patch_pairs.count(k) == 0 && patch_pairs.count(k2) == 0) {
+                        patch_pairs.emplace(k);
+                    }
+                }
             }
         }
     }
@@ -268,8 +288,8 @@ RulesCheckResult BoardRules::check_clearance_copper(const Board *brd, RulesCheck
     std::mutex patch_pair_mutex;
     for (size_t i = 0; i < std::thread::hardware_concurrency(); i++) {
         results.push_back(std::async(std::launch::async, clearance_cu_worker, std::ref(patch_pairs),
-                                     std::ref(patch_pair_mutex), std::ref(patches), brd, this, n_patch_pairs,
-                                     status_cb));
+                                     std::ref(patch_pair_mutex), std::ref(patches), std::ref(brd), std::ref(*this),
+                                     n_patch_pairs, status_cb));
     }
     for (auto &it : results) {
         auto res = it.get();
@@ -280,7 +300,7 @@ RulesCheckResult BoardRules::check_clearance_copper(const Board *brd, RulesCheck
     return r;
 }
 
-RulesCheckResult BoardRules::check_clearance_copper_non_copper(const Board *brd, RulesCheckCache &cache,
+RulesCheckResult BoardRules::check_clearance_copper_non_copper(const Board &brd, RulesCheckCache &cache,
                                                                check_status_cb_t status_cb) const
 {
     RulesCheckResult r;
@@ -295,9 +315,9 @@ RulesCheckResult BoardRules::check_clearance_copper_non_copper(const Board *brd,
         auto &patch_npth = patches.at(npth_key);
 
         for (const auto &it : patches) {
-            if (brd->get_layers().count(it.first.layer)
-                && brd->get_layers().at(it.first.layer).copper) { // need to check copper layer
-                Net *net = it.first.net ? &brd->block->nets.at(it.first.net) : nullptr;
+            if (brd.get_layers().count(it.first.layer)
+                && brd.get_layers().at(it.first.layer).copper) { // need to check copper layer
+                Net *net = it.first.net ? &brd.block->nets.at(it.first.net) : nullptr;
 
                 auto clearance = get_clearance_copper_other(net, it.first.layer)
                                          ->get_clearance(it.first.type, PatchType::HOLE_NPTH);
@@ -340,7 +360,7 @@ RulesCheckResult BoardRules::check_clearance_copper_non_copper(const Board *brd,
     // other cu
     std::set<int> layers;
     for (const auto &it : patches) { // collect copper layers
-        if (brd->get_layers().count(it.first.layer) && brd->get_layers().at(it.first.layer).copper) {
+        if (brd.get_layers().count(it.first.layer) && brd.get_layers().at(it.first.layer).copper) {
             layers.emplace(it.first.layer);
         }
     }
@@ -370,7 +390,7 @@ RulesCheckResult BoardRules::check_clearance_copper_non_copper(const Board *brd,
             auto p_non_other = it.second;
             if (!is_other(p_other.type))
                 std::swap(p_other, p_non_other);
-            Net *net = p_non_other.net ? &brd->block->nets.at(p_non_other.net) : nullptr;
+            Net *net = p_non_other.net ? &brd.block->nets.at(p_non_other.net) : nullptr;
 
             // figure out the clearance between this patch pair
             uint64_t clearance = 0;
@@ -431,9 +451,9 @@ RulesCheckResult BoardRules::check_clearance_copper_non_copper(const Board *brd,
     }
 
     for (const auto &it : patches) {
-        if (brd->get_layers().count(it.first.layer)
-            && brd->get_layers().at(it.first.layer).copper) { // need to check copper layer
-            Net *net = it.first.net ? &brd->block->nets.at(it.first.net) : nullptr;
+        if (brd.get_layers().count(it.first.layer)
+            && brd.get_layers().at(it.first.layer).copper) { // need to check copper layer
+            Net *net = it.first.net ? &brd.block->nets.at(it.first.net) : nullptr;
 
             auto clearance = get_clearance_copper_other(net, it.first.layer)
                                      ->get_clearance(it.first.type, PatchType::BOARD_EDGE);
@@ -474,7 +494,7 @@ RulesCheckResult BoardRules::check_clearance_copper_non_copper(const Board *brd,
     return r;
 }
 
-RulesCheckResult BoardRules::check_preflight(const Board *brd) const
+RulesCheckResult BoardRules::check_preflight(const Board &brd) const
 {
     RulesCheckResult r;
     r.level = RulesCheckErrorLevel::PASS;
@@ -483,8 +503,8 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
         return r;
     }
 
-    for (const auto &it_net : brd->airwires) {
-        const auto &net = brd->block->nets.at(it_net.first);
+    for (const auto &it_net : brd.airwires) {
+        const auto &net = brd.block->nets.at(it_net.first);
         for (const auto &it : it_net.second) {
             r.errors.emplace_back(RulesCheckErrorLevel::FAIL);
             auto &e = r.errors.back();
@@ -493,7 +513,7 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
             e.comment = "Airwire of net " + net.name;
         }
     }
-    for (const auto &it : brd->planes) {
+    for (const auto &it : brd.planes) {
         if (it.second.fragments.size() == 0) {
             r.errors.emplace_back(RulesCheckErrorLevel::FAIL);
             auto &e = r.errors.back();
@@ -502,7 +522,7 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
             e.comment = "Plane of net " + (it.second.net ? it.second.net->name : "No net") + " has no fragments";
         }
     }
-    for (const auto &it : brd->block->components) {
+    for (const auto &it : brd.block->components) {
         if (!it.second.part) {
             r.errors.emplace_back(RulesCheckErrorLevel::FAIL);
             auto &e = r.errors.back();
@@ -511,11 +531,11 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
         }
     }
     std::set<const Component *> unplaced_components;
-    std::transform(brd->block->components.begin(), brd->block->components.end(),
+    std::transform(brd.block->components.begin(), brd.block->components.end(),
                    std::inserter(unplaced_components, unplaced_components.begin()),
                    [](const auto &it) { return &it.second; });
 
-    for (const auto &it : brd->packages) {
+    for (const auto &it : brd.packages) {
         unplaced_components.erase(it.second.component);
     }
 
@@ -526,7 +546,7 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
         e.comment = "Component " + it->refdes + " is not placed";
     }
 
-    for (const auto &it : brd->tracks) {
+    for (const auto &it : brd.tracks) {
         auto &track = it.second;
         if (!track.net) {
             r.errors.emplace_back(RulesCheckErrorLevel::FAIL);
@@ -537,7 +557,7 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
         }
     }
 
-    for (const auto &it : brd->polygons) {
+    for (const auto &it : brd.polygons) {
         bool is_keepout = dynamic_cast<const Keepout *>(it.second.usage.ptr);
         bool is_plane = dynamic_cast<Plane *>(it.second.usage.ptr);
         if (BoardLayers::is_copper(it.second.layer) && !(is_plane || is_keepout)) {
@@ -547,11 +567,11 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
             if (e.has_location)
                 e.location = it.second.vertices.front().position;
             e.comment =
-                    "Polygon on layer " + brd->get_layers().at(it.second.layer).name + " is not a keepout or a plane";
+                    "Polygon on layer " + brd.get_layers().at(it.second.layer).name + " is not a keepout or a plane";
         }
     }
 
-    for (const auto &it : brd->lines) {
+    for (const auto &it : brd.lines) {
         if (it.second.layer == BoardLayers::L_OUTLINE) {
             r.errors.emplace_back(RulesCheckErrorLevel::FAIL);
             auto &e = r.errors.back();
@@ -562,7 +582,7 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
         }
     }
 
-    for (const auto &it : brd->arcs) {
+    for (const auto &it : brd.arcs) {
         if (it.second.layer == BoardLayers::L_OUTLINE) {
             r.errors.emplace_back(RulesCheckErrorLevel::FAIL);
             auto &e = r.errors.back();
@@ -578,7 +598,7 @@ RulesCheckResult BoardRules::check_preflight(const Board *brd) const
 }
 
 
-RulesCheckResult BoardRules::check_clearance_copper_keepout(const Board *brd, RulesCheckCache &cache,
+RulesCheckResult BoardRules::check_clearance_copper_keepout(const Board &brd, RulesCheckCache &cache,
                                                             check_status_cb_t status_cb) const
 {
     RulesCheckResult r;
@@ -589,11 +609,11 @@ RulesCheckResult BoardRules::check_clearance_copper_keepout(const Board *brd, Ru
     std::set<int> layers;
     const auto &patches = c->get_canvas()->get_patches();
     for (const auto &it : patches) { // collect copper layers
-        if (brd->get_layers().count(it.first.layer) && brd->get_layers().at(it.first.layer).copper) {
+        if (brd.get_layers().count(it.first.layer) && brd.get_layers().at(it.first.layer).copper) {
             layers.emplace(it.first.layer);
         }
     }
-    auto keepout_contours = brd->get_keepout_contours();
+    auto keepout_contours = brd.get_keepout_contours();
     auto n_patches = patches.size();
     auto n_keepouts = keepout_contours.size();
     int i_keepout = 0;
@@ -614,7 +634,7 @@ RulesCheckResult BoardRules::check_clearance_copper_keepout(const Board *brd, Ru
                 && (it.first.layer == keepout->polygon->layer || keepout->all_cu_layers)
                 && keepout->patch_types_cu.count(it.first.type)) {
 
-                const Net *net = it.first.net ? &brd->block->nets.at(it.first.net) : nullptr;
+                const Net *net = it.first.net ? &brd.block->nets.at(it.first.net) : nullptr;
                 int64_t clearance = 0;
                 for (const auto &rule : rules) {
                     if (rule->match.match(net)
@@ -666,7 +686,90 @@ RulesCheckResult BoardRules::check_clearance_copper_keepout(const Board *brd, Ru
     return r;
 }
 
-RulesCheckResult BoardRules::check(RuleID id, const Board *brd, RulesCheckCache &cache,
+RulesCheckResult BoardRules::check_clearance_same_net(const Board &brd, RulesCheckCache &cache,
+                                                      check_status_cb_t status_cb) const
+{
+    RulesCheckResult r;
+    r.level = RulesCheckErrorLevel::PASS;
+    status_cb("Getting patches");
+    auto rules = get_rules_sorted<RuleClearanceSameNet>(RuleID::CLEARANCE_SAME_NET);
+    auto c = dynamic_cast<RulesCheckCacheBoardImage *>(cache.get_cache(RulesCheckCacheID::BOARD_IMAGE));
+    std::set<int> layers;
+    const auto &patches = c->get_canvas()->get_patches();
+    for (const auto &it : patches) { // collect copper layers
+        if (brd.get_layers().count(it.first.layer) && brd.get_layers().at(it.first.layer).copper) {
+            layers.emplace(it.first.layer);
+        }
+    }
+    status_cb("Building patch pairs");
+
+    static const std::set<PatchType> patch_types = {PatchType::PAD, PatchType::PAD_TH, PatchType::VIA,
+                                                    PatchType::HOLE_NPTH};
+
+    for (const auto layer : layers) { // check each layer individually
+        // assemble a list of patch pairs we'll need to check
+        std::set<std::pair<CanvasPatch::PatchKey, CanvasPatch::PatchKey>> patch_pairs;
+        for (const auto &it : patches) {
+            for (const auto &it_other : patches) {
+                if (layer == it.first.layer && it.first.layer == it_other.first.layer
+                    && patch_types.count(it.first.type) && patch_types.count(it_other.first.type)
+                    && it.first.type != it_other.first.type && it.first.net == it_other.first.net) {
+                    std::pair<CanvasPatch::PatchKey, CanvasPatch::PatchKey> k = {it.first, it_other.first};
+                    auto k2 = k;
+                    std::swap(k2.first, k2.second);
+                    if (patch_pairs.count(k) == 0 && patch_pairs.count(k2) == 0) {
+                        patch_pairs.emplace(k);
+                    }
+                }
+            }
+        }
+
+        for (const auto &[p1, p2] : patch_pairs) {
+            const Net *net = p1.net ? &brd.block->nets.at(p1.net) : nullptr;
+            auto rule = get_clearance_same_net(net, layer);
+            auto clearance = rule->get_clearance(p1.type, p2.type);
+            if (clearance >= 0) {
+                // expand p1 patch by clearance
+                ClipperLib::ClipperOffset ofs;
+                ofs.ArcTolerance = 10e3;
+                ofs.AddPaths(patches.at(p1), ClipperLib::jtRound, ClipperLib::etClosedPolygon);
+                ClipperLib::Paths paths_ofs;
+                ofs.Execute(paths_ofs, clearance);
+
+                // intersect expanded and this patch
+                ClipperLib::Clipper clipper;
+                clipper.AddPaths(paths_ofs, ClipperLib::ptClip, true);
+                clipper.AddPaths(patches.at(p2), ClipperLib::ptSubject, true);
+                ClipperLib::Paths errors;
+                clipper.Execute(ClipperLib::ctIntersection, errors, ClipperLib::pftNonZero, ClipperLib::pftNonZero);
+
+                // no intersection: no clearance violation
+                if (errors.size() > 0) {
+                    for (const auto &ite : errors) {
+                        r.errors.emplace_back(RulesCheckErrorLevel::FAIL);
+                        auto &e = r.errors.back();
+                        e.has_location = true;
+                        Accumulator<Coordi> acc;
+                        for (const auto &ite2 : ite) {
+                            acc.accumulate({ite2.X, ite2.Y});
+                        }
+                        e.location = acc.get();
+                        e.comment = patch_type_names.at(p1.type) + " near " + patch_type_names.at(p2.type) + " "
+                                    + get_net_name(net) + " on layer " + brd.get_layers().at(layer).name;
+                        e.error_polygons = {ite};
+                    }
+                }
+            }
+        }
+    }
+
+
+    r.update();
+    return r;
+}
+
+
+RulesCheckResult BoardRules::check(RuleID id, const Board &brd, RulesCheckCache &cache,
                                    check_status_cb_t status_cb) const
 {
     switch (id) {
@@ -688,6 +791,9 @@ RulesCheckResult BoardRules::check(RuleID id, const Board *brd, RulesCheckCache
     case RuleID::PREFLIGHT_CHECKS:
         return BoardRules::check_preflight(brd);
 
+    case RuleID::CLEARANCE_SAME_NET:
+        return BoardRules::check_clearance_same_net(brd, cache, status_cb);
+
     default:
         return RulesCheckResult();
     }
