@@ -15,8 +15,16 @@ std::pair<float, Coordf> CanvasGL::get_scale_and_offset()
     return std::make_pair(scale, offset);
 }
 
+bool CanvasGL::can_set_scale() const
+{
+    return !(pan_dragging || zoom_animator.is_running() || gesture_zoom->is_recognized()
+             || gesture_drag->is_recognized());
+}
+
 void CanvasGL::set_scale_and_offset(float sc, Coordf ofs)
 {
+    if (!can_set_scale())
+        return;
     scale = sc;
     offset = ofs;
     update_viewmat();
@@ -324,26 +332,31 @@ Coordi CanvasGL::snap_to_grid(const Coordi &c, unsigned int div) const
     return grid.origin + Coordi(xi, yi);
 }
 
+int CanvasGL::get_last_grid_div() const
+{
+    return last_grid_div;
+}
+
 void CanvasGL::cursor_move(GdkEvent *motion_event)
 {
     gdouble x, y;
     gdk_event_get_coords(motion_event, &x, &y);
     cursor_pos = screen2canvas(Coordf(x, y));
 
+    GdkModifierType state;
+    gdk_event_get_state(motion_event, &state);
+    last_grid_div = 1;
+    if (state & grid_fine_modifier) {
+        last_grid_div = 10;
+    }
+
     if (cursor_external) {
         Coordi c(cursor_pos.x, cursor_pos.y);
         s_signal_cursor_moved.emit(c);
         return;
     }
 
-    GdkModifierType state;
-    gdk_event_get_state(motion_event, &state);
-    unsigned int div = 1;
-    if (state & grid_fine_modifier) {
-        div = 10;
-    }
-
-    Coordi t = snap_to_grid(Coordi(cursor_pos.x, cursor_pos.y), div);
+    Coordi t = snap_to_grid(Coordi(cursor_pos.x, cursor_pos.y), last_grid_div);
 
     const auto &f = std::find_if(targets.begin(), targets.end(), [t, this](const auto &a) -> bool {
         return a.p == t && this->layer_is_visible(a.layer) && can_snap_to_target(a);
@@ -431,6 +444,12 @@ void CanvasGL::zoom_to_bbox(const std::pair<Coordf, Coordf> &bb)
     zoom_to_bbox(bb.first, bb.second);
 }
 
+void CanvasGL::zoom_to_center(float factor)
+{
+    if (!can_set_scale())
+        return;
+    set_scale(m_width / 2, m_height / 2, scale * factor);
+}
 
 void CanvasGL::ensure_min_size(float w, float h)
 {
@@ -743,6 +762,14 @@ bool CanvasGL::layer_is_visible(int layer) const
         return layer == work_layer || layer >= 10000;
 }
 
+bool CanvasGL::layer_is_visible(LayerRange layer) const
+{
+    if (layer_mode == LayerMode::AS_IS)
+        return Canvas::layer_is_visible(layer);
+    else
+        return layer.overlaps(work_layer) || layer.start() >= 10000;
+}
+
 void CanvasGL::set_colors2(const std::vector<ColorI> &c)
 {
     colors2 = c;
