@@ -100,7 +100,7 @@ public:
         return grid.mul;
     }
 
-    typedef sigc::signal<std::string, ObjectType, UUID> type_signal_request_display_name;
+    typedef sigc::signal<std::string, const SelectableRef &> type_signal_request_display_name;
     type_signal_request_display_name signal_request_display_name()
     {
         return s_signal_request_display_name;
@@ -122,6 +122,7 @@ public:
     void zoom_to_bbox(const Coordf &a, const Coordf &b);
     void zoom_to_bbox(const std::pair<Coordf, Coordf> &bb);
     void ensure_min_size(float w, float h);
+    void zoom_to_center(float factor);
 
     Glib::PropertyProxy<int> property_work_layer()
     {
@@ -151,6 +152,8 @@ public:
     bool smooth_zoom = true;
     bool snap_to_targets = true;
 
+    float zoom_base = 1.5;
+
     void inhibit_drag_selection();
 
     int _animate_step(GdkFrameClock *frame_clock);
@@ -178,9 +181,12 @@ public:
     SelectionFilter selection_filter;
 
     bool layer_is_visible(int layer) const;
+    bool layer_is_visible(LayerRange layer) const;
 
     void set_colors2(const std::vector<ColorI> &c);
 
+    int get_last_grid_div() const;
+
 protected:
     void push() override;
     void request_push() override;
@@ -295,6 +301,10 @@ private:
 
     std::vector<ColorI> colors2;
 
+    bool can_set_scale() const;
+
+    int last_grid_div = 1;
+
 protected:
     void on_size_allocate(Gtk::Allocation &alloc) override;
     void on_realize() override;
