@@ -153,8 +153,15 @@ void FaceRenderer::render()
         glUniform1f(highlight_intensity_loc, ca.highlight_intensity);
 
         for (const auto &it : ca.models) {
-            if (ca.package_transform_idxs.count(it.first)) {
-                auto idxs = ca.package_transform_idxs.at(it.first);
+            std::pair<std::string, bool> populate = {it.first, false};
+            std::pair<std::string, bool> nopopulate = {it.first, true};
+            if (ca.package_transform_idxs.count(populate)) {
+                auto idxs = ca.package_transform_idxs.at(populate);
+                glDrawElementsInstancedBaseInstance(GL_TRIANGLES, it.second.second, GL_UNSIGNED_INT,
+                                                    (void *)(it.second.first * sizeof(int)), idxs.second, idxs.first);
+            }
+            if (ca.show_dnp_models && ca.package_transform_idxs.count(nopopulate)) {
+                auto idxs = ca.package_transform_idxs.at(nopopulate);
                 glDrawElementsInstancedBaseInstance(GL_TRIANGLES, it.second.second, GL_UNSIGNED_INT,
                                                     (void *)(it.second.first * sizeof(int)), idxs.second, idxs.first);
             }
