@@ -56,7 +56,10 @@ enum class ObjectType {
     CONNECTION_LINE,
     AIRWIRE,
     BOARD_PANEL,
-    PICTURE
+    PICTURE,
+    DECAL,
+    BOARD_DECAL,
+    PROJECT,
 };
 enum class PatchType { OTHER, TRACK, PAD, PAD_TH, VIA, PLANE, HOLE_PTH, HOLE_NPTH, BOARD_EDGE, TEXT, N_TYPES };
 
