@@ -1,14 +1,11 @@
 #pragma once
 #include "util/uuid.hpp"
 #include "common.hpp"
-#include "util/uuid_provider.hpp"
 #include "block/net.hpp"
 #include "util/uuid_ptr.hpp"
 #include "nlohmann/json_fwd.hpp"
 #include "block/bus.hpp"
-#include <vector>
-#include <map>
-#include <fstream>
+#include "util/layer_range.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
@@ -22,7 +19,7 @@ using json = nlohmann::json;
  * When used on a Board or a Sheet, a Junction may get assigned a Net
  * or a Bus and a net segment.
  */
-class Junction : public UUIDProvider {
+class Junction {
 public:
     Junction(const UUID &uu, const json &j);
     Junction(const UUID &uu);
@@ -31,14 +28,14 @@ public:
     Coord<int64_t> position;
 
 
-    virtual UUID get_uuid() const;
+    UUID get_uuid() const;
 
     // not stored
     uuid_ptr<Net> net = nullptr;
     uuid_ptr<Bus> bus = nullptr;
     UUID net_segment = UUID();
     bool warning = false;
-    int layer = 10000;
+    LayerRange layer = 10000;
     bool needs_via = false;
     bool has_via = false; // in schematic: true if has net label, etc.
     unsigned int connection_count = 0;
