@@ -1,14 +1,12 @@
 #include "core_frame.hpp"
 #include "util/util.hpp"
-#include <algorithm>
-#include <memory>
 #include "nlohmann/json.hpp"
 #include <giomm/file.h>
 #include <glibmm/fileutils.h>
 
 namespace horizon {
-CoreFrame::CoreFrame(const std::string &frame_filename)
-    : frame(Frame::new_from_file(frame_filename)), m_frame_filename(frame_filename)
+CoreFrame::CoreFrame(const std::string &frame_filename, IPool &pool)
+    : Core(pool), frame(Frame::new_from_file(frame_filename)), m_frame_filename(frame_filename)
 {
     rebuild();
 }
@@ -60,9 +58,9 @@ void CoreFrame::rebuild(bool from_undo)
     Core::rebuild(from_undo);
 }
 
-LayerProvider *CoreFrame::get_layer_provider()
+LayerProvider &CoreFrame::get_layer_provider()
 {
-    return &frame;
+    return frame;
 }
 
 const Frame *CoreFrame::get_canvas_data() const
@@ -70,9 +68,9 @@ const Frame *CoreFrame::get_canvas_data() const
     return &frame;
 }
 
-Frame *CoreFrame::get_frame()
+Frame &CoreFrame::get_frame()
 {
-    return &frame;
+    return frame;
 }
 
 CoreFrame::HistoryItem::HistoryItem(const Frame &fr) : frame(fr)
