@@ -1,20 +1,18 @@
 #pragma once
 #include "frame/frame.hpp"
 #include "core.hpp"
-#include <iostream>
-#include <memory>
 #include "nlohmann/json.hpp"
 #include "document/idocument_frame.hpp"
 
 namespace horizon {
 class CoreFrame : public Core, public IDocumentFrame {
 public:
-    CoreFrame(const std::string &frame_filename);
+    CoreFrame(const std::string &frame_filename, IPool &pool);
     bool has_object_type(ObjectType ty) const override;
 
-    Frame *get_frame() override;
+    Frame &get_frame() override;
     const Frame *get_canvas_data() const;
-    class LayerProvider *get_layer_provider() override;
+    class LayerProvider &get_layer_provider() override;
 
     void rebuild(bool from_undo = false) override;
 
@@ -27,6 +25,11 @@ public:
         return ObjectType::FRAME;
     }
 
+    const FileVersion &get_version() const override
+    {
+        return frame.version;
+    }
+
 private:
     std::map<UUID, Polygon> *get_polygon_map() override;
     std::map<UUID, Junction> *get_junction_map() override;
