@@ -1,24 +1,22 @@
 #include "core_package.hpp"
 #include "core_properties.hpp"
-#include <algorithm>
-#include <fstream>
 #include "nlohmann/json.hpp"
 #include "logger/logger.hpp"
 #include "util/util.hpp"
 #include <giomm/file.h>
 #include <glibmm/fileutils.h>
 #include <glibmm/miscutils.h>
+#include "pool/ipool.hpp"
 
 namespace horizon {
-CorePackage::CorePackage(const std::string &filename, Pool &pool)
-    : package(Package::new_from_file(filename, pool)), m_filename(filename),
+CorePackage::CorePackage(const std::string &filename, IPool &pool)
+    : Core(pool), package(Package::new_from_file(filename, pool)), m_filename(filename),
       m_pictures_dir(Glib::build_filename(Glib::path_get_dirname(filename), "pictures")), rules(package.rules),
       parameter_program_code(package.parameter_program.get_code()), parameter_set(package.parameter_set),
       models(package.models), default_model(package.default_model)
 {
     package.load_pictures(m_pictures_dir);
     rebuild();
-    m_pool = &pool;
 }
 
 bool CorePackage::has_object_type(ObjectType ty) const
@@ -44,14 +42,14 @@ bool CorePackage::has_object_type(ObjectType ty) const
     return false;
 }
 
-LayerProvider *CorePackage::get_layer_provider()
+LayerProvider &CorePackage::get_layer_provider()
 {
-    return &package;
+    return package;
 }
 
-Package *CorePackage::get_package()
+Package &CorePackage::get_package()
 {
-    return &package;
+    return package;
 }
 
 Rules *CorePackage::get_rules()
@@ -241,8 +239,8 @@ const Package *CorePackage::get_canvas_data()
 
 void CorePackage::reload_pool()
 {
-    m_pool->clear();
-    package.update_refs(*m_pool);
+    m_pool.clear();
+    package.update_refs(m_pool);
     history_clear();
     rebuild();
 }
@@ -263,6 +261,7 @@ void CorePackage::save(const std::string &suffix)
     package.parameter_program.set_code(parameter_program_code);
     package.models = models;
     package.default_model = default_model;
+    package.rules = rules;
 
     s_signal_save.emit();
 
