@@ -1,24 +1,19 @@
 #pragma once
-#include "common/layer.hpp"
 #include "core.hpp"
 #include "pool/padstack.hpp"
-#include "pool/pool.hpp"
-#include <deque>
-#include <iostream>
-#include <memory>
 #include "document/idocument_padstack.hpp"
 
 namespace horizon {
 class CorePadstack : public Core, public virtual IDocumentPadstack {
 public:
-    CorePadstack(const std::string &filename, Pool &pool);
+    CorePadstack(const std::string &filename, IPool &pool);
     bool has_object_type(ObjectType ty) const override;
 
-    class LayerProvider *get_layer_provider() override;
+    class LayerProvider &get_layer_provider() override;
 
     void rebuild(bool from_undo = false) override;
 
-    Padstack *get_padstack() override;
+    Padstack &get_padstack() override;
 
     bool set_property(ObjectType type, const UUID &uu, ObjectProperty::ID property,
                       const class PropertyValue &value) override;
@@ -39,6 +34,11 @@ public:
         return ObjectType::PADSTACK;
     }
 
+    const FileVersion &get_version() const override
+    {
+        return padstack.version;
+    }
+
 private:
     std::map<UUID, Polygon> *get_polygon_map() override;
     std::map<UUID, Hole> *get_hole_map() override;
