@@ -1,18 +1,15 @@
 #pragma once
 #include "block/block.hpp"
 #include "core.hpp"
-#include "pool/pool.hpp"
-#include "pool/symbol.hpp"
 #include "schematic/schematic.hpp"
 #include "document/idocument_schematic.hpp"
-#include <iostream>
 #include <memory>
 
 namespace horizon {
 class CoreSchematic : public Core, public virtual IDocumentSchematic {
 public:
     CoreSchematic(const std::string &schematic_filename, const std::string &block_filename,
-                  const std::string &pictures_dir, Pool &pool);
+                  const std::string &pictures_dir, IPool &pool);
     bool has_object_type(ObjectType ty) const override;
 
     Junction *get_junction(const UUID &uu) override;
@@ -49,7 +46,7 @@ public:
     std::vector<NetLabel *> get_net_labels() override;
 
     class Block *get_block() override;
-    class LayerProvider *get_layer_provider() override;
+    class LayerProvider &get_layer_provider() override;
 
     bool set_property(ObjectType type, const UUID &uu, ObjectProperty::ID property,
                       const class PropertyValue &value) override;
@@ -94,6 +91,11 @@ public:
         return ObjectType::SCHEMATIC;
     }
 
+    const FileVersion &get_version() const override
+    {
+        return sch.version;
+    }
+
 private:
     Block block;
     const bool project_meta_loaded_from_block;
