@@ -6,12 +6,12 @@
 #include "util/util.hpp"
 #include <giomm/file.h>
 #include <glibmm/fileutils.h>
+#include "pool/ipool.hpp"
 
 namespace horizon {
-CoreSymbol::CoreSymbol(const std::string &filename, Pool &pool)
-    : sym(Symbol::new_from_file(filename, pool)), m_filename(filename)
+CoreSymbol::CoreSymbol(const std::string &filename, IPool &pool)
+    : Core(pool), sym(Symbol::new_from_file(filename, pool)), m_filename(filename)
 {
-    m_pool = &pool;
     rebuild();
 }
 
@@ -34,6 +34,11 @@ bool CoreSymbol::has_object_type(ObjectType ty) const
     return false;
 }
 
+Rules *CoreSymbol::get_rules()
+{
+    return &rules;
+}
+
 std::map<UUID, Text> *CoreSymbol::get_text_map()
 {
     return &sym.texts;
@@ -52,9 +57,9 @@ Line *CoreSymbol::get_line(const UUID &uu)
 {
     return &sym.lines.at(uu);
 }
-SymbolPin *CoreSymbol::get_symbol_pin(const UUID &uu)
+SymbolPin &CoreSymbol::get_symbol_pin(const UUID &uu)
 {
-    return sym.get_symbol_pin(uu);
+    return sym.pins.at(uu);
 }
 Arc *CoreSymbol::get_arc(const UUID &uu)
 {
@@ -95,10 +100,9 @@ void CoreSymbol::delete_symbol_pin(const UUID &uu)
 {
     sym.pins.erase(uu);
 }
-SymbolPin *CoreSymbol::insert_symbol_pin(const UUID &uu)
+SymbolPin &CoreSymbol::insert_symbol_pin(const UUID &uu)
 {
-    auto x = sym.pins.emplace(std::make_pair(uu, uu));
-    return &(x.first->second);
+    return sym.pins.emplace(uu, uu).first->second;
 }
 
 std::vector<Line *> CoreSymbol::get_lines()
@@ -119,15 +123,6 @@ std::vector<Arc *> CoreSymbol::get_arcs()
     return r;
 }
 
-std::vector<const Pin *> CoreSymbol::get_pins()
-{
-    std::vector<const Pin *> r;
-    for (auto &it : sym.unit->pins) {
-        r.push_back(&it.second);
-    }
-    return r;
-}
-
 bool CoreSymbol::get_property(ObjectType type, const UUID &uu, ObjectProperty::ID property, PropertyValue &value)
 {
     if (Core::get_property(type, uu, property, value))
@@ -248,7 +243,7 @@ std::string CoreSymbol::get_display_name(ObjectType type, const UUID &uu)
 {
     switch (type) {
     case ObjectType::SYMBOL_PIN:
-        return get_symbol_pin(uu)->name;
+        return get_symbol_pin(uu).name;
 
     default:
         return Core::get_display_name(type, uu);
@@ -274,9 +269,9 @@ void CoreSymbol::history_load(unsigned int i)
     s_signal_rebuilt.emit();
 }
 
-LayerProvider *CoreSymbol::get_layer_provider()
+LayerProvider &CoreSymbol::get_layer_provider()
 {
-    return &sym;
+    return sym;
 }
 
 const Symbol *CoreSymbol::get_canvas_data()
@@ -296,15 +291,15 @@ std::pair<Coordi, Coordi> CoreSymbol::get_bbox()
     return bb;
 }
 
-Symbol *CoreSymbol::get_symbol()
+Symbol &CoreSymbol::get_symbol()
 {
-    return &sym;
+    return sym;
 }
 
 void CoreSymbol::reload_pool()
 {
-    m_pool->clear();
-    sym.unit = m_pool->get_unit(sym.unit.uuid);
+    m_pool.clear();
+    sym.unit = m_pool.get_unit(sym.unit.uuid);
     history_clear();
     rebuild();
 }
