@@ -1,6 +1,5 @@
 #pragma once
 #include "core.hpp"
-#include "pool/pool.hpp"
 #include "pool/symbol.hpp"
 #include <deque>
 #include <iostream>
@@ -10,13 +9,13 @@
 namespace horizon {
 class CoreSymbol : public Core, public IDocumentSymbol {
 public:
-    CoreSymbol(const std::string &filename, Pool &pool);
+    CoreSymbol(const std::string &filename, IPool &pool);
     bool has_object_type(ObjectType ty) const override;
-    Symbol *get_symbol() override;
+    Symbol &get_symbol() override;
 
     Junction *get_junction(const UUID &uu) override;
     Line *get_line(const UUID &uu) override;
-    SymbolPin *get_symbol_pin(const UUID &uu) override;
+    SymbolPin &get_symbol_pin(const UUID &uu) override;
     Arc *get_arc(const UUID &uu) override;
 
     Junction *insert_junction(const UUID &uu) override;
@@ -26,14 +25,13 @@ public:
     Arc *insert_arc(const UUID &uu) override;
     void delete_arc(const UUID &uu) override;
 
-    SymbolPin *insert_symbol_pin(const UUID &uu) override;
+    SymbolPin &insert_symbol_pin(const UUID &uu) override;
     void delete_symbol_pin(const UUID &uu) override;
 
-    class LayerProvider *get_layer_provider() override;
+    class LayerProvider &get_layer_provider() override;
 
     std::vector<Line *> get_lines() override;
     std::vector<Arc *> get_arcs() override;
-    std::vector<const Pin *> get_pins() override;
 
     void rebuild(bool from_undo = false) override;
 
@@ -60,6 +58,14 @@ public:
         return ObjectType::SYMBOL;
     }
 
+    class Rules *get_rules() override;
+
+    const FileVersion &get_version() const override
+    {
+        return sym.version;
+    }
+
+
 private:
     std::map<UUID, Text> *get_text_map() override;
     std::map<UUID, Polygon> *get_polygon_map() override;
@@ -68,6 +74,8 @@ private:
     std::string m_filename;
     Symbol::PinDisplayMode pin_display_mode = Symbol::PinDisplayMode::PRIMARY;
 
+    SymbolRules rules;
+
     class HistoryItem : public Core::HistoryItem {
     public:
         HistoryItem(const Symbol &s) : sym(s)
