@@ -47,7 +47,7 @@ void ToolHelperMove::move_do(const Coordi &delta)
             doc.r->get_hole(it.uuid)->placement.shift += delta;
             break;
         case ObjectType::SYMBOL_PIN:
-            doc.y->get_symbol_pin(it.uuid)->position += delta;
+            doc.y->get_symbol_pin(it.uuid).position += delta;
             break;
         case ObjectType::SCHEMATIC_SYMBOL:
             doc.c->get_schematic_symbol(it.uuid)->placement.shift += delta;
@@ -56,7 +56,7 @@ void ToolHelperMove::move_do(const Coordi &delta)
             doc.b->get_board()->packages.at(it.uuid).placement.shift += delta;
             break;
         case ObjectType::PAD:
-            doc.k->get_package()->pads.at(it.uuid).placement.shift += delta;
+            doc.k->get_package().pads.at(it.uuid).placement.shift += delta;
             break;
         case ObjectType::BOARD_HOLE:
             doc.b->get_board()->holes.at(it.uuid).placement.shift += delta;
@@ -71,7 +71,7 @@ void ToolHelperMove::move_do(const Coordi &delta)
             doc.r->get_polygon(it.uuid)->vertices.at(it.vertex).arc_center += delta;
             break;
         case ObjectType::SHAPE:
-            doc.a->get_padstack()->shapes.at(it.uuid).placement.shift += delta;
+            doc.a->get_padstack().shapes.at(it.uuid).placement.shift += delta;
             break;
         case ObjectType::BOARD_PANEL:
             doc.b->get_board()->board_panels.at(it.uuid).placement.shift += delta;
@@ -79,6 +79,9 @@ void ToolHelperMove::move_do(const Coordi &delta)
         case ObjectType::PICTURE:
             doc.r->get_picture(it.uuid)->placement.shift += delta;
             break;
+        case ObjectType::BOARD_DECAL:
+            doc.b->get_board()->decals.at(it.uuid).placement.shift += delta;
+            break;
         case ObjectType::DIMENSION: {
             auto dim = doc.r->get_dimension(it.uuid);
             if (it.vertex < 2) {
@@ -207,9 +210,9 @@ void ToolHelperMove::move_mirror_or_rotate(const Coordi &center, bool rotate)
             break;
 
         case ObjectType::SYMBOL_PIN: {
-            SymbolPin *pin = doc.y->get_symbol_pin(it.uuid);
-            transform(pin->position, center, rotate);
-            pin->orientation = transform_orientation(pin->orientation, rotate);
+            auto &pin = doc.y->get_symbol_pin(it.uuid);
+            transform(pin.position, center, rotate);
+            pin.orientation = transform_orientation(pin.orientation, rotate);
         } break;
 
         case ObjectType::BUS_RIPPER: {
@@ -307,10 +310,10 @@ void ToolHelperMove::move_mirror_or_rotate(const Coordi &center, bool rotate)
         } break;
 
         case ObjectType::PAD: {
-            Pad *pad = &doc.k->get_package()->pads.at(it.uuid);
-            transform(pad->placement.shift, center, rotate);
+            auto &pad = doc.k->get_package().pads.at(it.uuid);
+            transform(pad.placement.shift, center, rotate);
             if (rotate) {
-                pad->placement.inc_angle_deg(-90);
+                pad.placement.inc_angle_deg(-90);
             }
         } break;
 
@@ -339,10 +342,23 @@ void ToolHelperMove::move_mirror_or_rotate(const Coordi &center, bool rotate)
         } break;
 
         case ObjectType::SHAPE: {
-            Shape *shape = &doc.a->get_padstack()->shapes.at(it.uuid);
-            transform(shape->placement.shift, center, rotate);
+            auto &shape = doc.a->get_padstack().shapes.at(it.uuid);
+            transform(shape.placement.shift, center, rotate);
             if (rotate) {
-                shape->placement.inc_angle_deg(-90);
+                shape.placement.inc_angle_deg(-90);
+            }
+        } break;
+
+        case ObjectType::BOARD_DECAL: {
+            auto &decal = doc.b->get_board()->decals.at(it.uuid);
+            transform(decal.placement.shift, center, rotate);
+            if (rotate) {
+                decal.placement.inc_angle_deg(-90);
+            }
+            else {
+                decal.flip ^= true;
+                decal.placement.invert_angle();
+                doc.b->get_board()->expand_decals();
             }
         } break;
 
