@@ -7,9 +7,9 @@ namespace horizon {
 class ToolImportDXF : public ToolHelperMove {
 public:
     ToolImportDXF(IDocument *c, ToolID tid);
-    ToolResponse begin(const ToolArgs &args);
-    ToolResponse update(const ToolArgs &args);
-    bool can_begin();
+    ToolResponse begin(const ToolArgs &args) override;
+    ToolResponse update(const ToolArgs &args) override;
+    bool can_begin() override;
     std::set<InToolActionID> get_actions() const override
     {
         using I = InToolActionID;
