@@ -1,6 +1,6 @@
 #include "tool_import_kicad_package.hpp"
 #include "document/idocument_package.hpp"
-#include "pool/package.hpp"
+#include "document/idocument_decal.hpp"
 #include "imp/imp_interface.hpp"
 #include "util/kicad_package_parser.hpp"
 #include "sexpr/sexpr_parser.h"
@@ -16,7 +16,7 @@ ToolImportKiCadPackage::ToolImportKiCadPackage(IDocument *c, ToolID tid) : ToolB
 
 bool ToolImportKiCadPackage::can_begin()
 {
-    return doc.k;
+    return doc.k || doc.d;
 }
 
 static std::string slurp_from_file(const std::string &filename)
@@ -36,8 +36,17 @@ ToolResponse ToolImportKiCadPackage::begin(const ToolArgs &args)
     if (auto r = imp->dialogs.ask_kicad_package_filename()) {
         SEXPR::PARSER parser;
         std::unique_ptr<SEXPR::SEXPR> sexpr_data(parser.Parse(slurp_from_file(*r)));
-        KiCadPackageParser kp(*doc.k->get_package(), doc.k->get_pool());
-        kp.parse(sexpr_data.get());
+        if (doc.k) {
+            KiCadPackageParser kp(doc.k->get_package(), doc.k->get_pool());
+            kp.parse(sexpr_data.get());
+        }
+        else if (doc.d) {
+            KiCadDecalParser kp(doc.d->get_decal());
+            kp.parse(sexpr_data.get());
+        }
+        else {
+            return ToolResponse::end();
+        }
         return ToolResponse::commit();
     }
     else {
