@@ -6,9 +6,9 @@ namespace horizon {
 class ToolImportKiCadPackage : public ToolBase {
 public:
     ToolImportKiCadPackage(IDocument *c, ToolID tid);
-    ToolResponse begin(const ToolArgs &args);
-    ToolResponse update(const ToolArgs &args);
-    bool can_begin();
+    ToolResponse begin(const ToolArgs &args) override;
+    ToolResponse update(const ToolArgs &args) override;
+    bool can_begin() override;
 
 private:
 };
