@@ -156,7 +156,7 @@ Coordi ToolMove::get_selection_center()
             accu.accumulate(doc.b->get_board()->holes.at(it.uuid).placement.shift);
             break;
         case ObjectType::SYMBOL_PIN:
-            accu.accumulate(doc.y->get_symbol_pin(it.uuid)->position);
+            accu.accumulate(doc.y->get_symbol_pin(it.uuid).position);
             break;
         case ObjectType::SCHEMATIC_SYMBOL:
             accu.accumulate(doc.c->get_schematic_symbol(it.uuid)->placement.shift);
@@ -165,7 +165,7 @@ Coordi ToolMove::get_selection_center()
             accu.accumulate(doc.b->get_board()->packages.at(it.uuid).placement.shift);
             break;
         case ObjectType::PAD:
-            accu.accumulate(doc.k->get_package()->pads.at(it.uuid).placement.shift);
+            accu.accumulate(doc.k->get_package().pads.at(it.uuid).placement.shift);
             break;
         case ObjectType::TEXT:
             accu.accumulate(doc.r->get_text(it.uuid)->placement.shift);
@@ -183,7 +183,7 @@ Coordi ToolMove::get_selection_center()
             accu.accumulate(doc.r->get_polygon(it.uuid)->vertices.at(it.vertex).arc_center);
             break;
         case ObjectType::SHAPE:
-            accu.accumulate(doc.a->get_padstack()->shapes.at(it.uuid).placement.shift);
+            accu.accumulate(doc.a->get_padstack().shapes.at(it.uuid).placement.shift);
             break;
         case ObjectType::BOARD_PANEL:
             accu.accumulate(doc.b->get_board()->board_panels.at(it.uuid).placement.shift);
@@ -191,6 +191,9 @@ Coordi ToolMove::get_selection_center()
         case ObjectType::PICTURE:
             accu.accumulate(doc.r->get_picture(it.uuid)->placement.shift);
             break;
+        case ObjectType::BOARD_DECAL:
+            accu.accumulate(doc.b->get_board()->decals.at(it.uuid).placement.shift);
+            break;
         default:;
         }
     }
