@@ -56,7 +56,7 @@ private:
 
 void ToolPaste::fix_layer(int &la)
 {
-    if (doc.r->get_layer_provider()->get_layers().count(la) == 0) {
+    if (doc.r->get_layer_provider().get_layers().count(la) == 0) {
         la = 0;
     }
 }
@@ -147,15 +147,15 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
     if (j.count("pads") && doc.k) {
         const json &o = j["pads"];
         std::vector<Pad *> pads;
-        int max_name = doc.k->get_package()->get_max_pad_name();
+        int max_name = doc.k->get_package().get_max_pad_name();
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
-            auto &x = doc.k->get_package()->pads.emplace(u, Pad(u, it.value(), *doc.r->get_pool())).first->second;
+            auto &x = doc.k->get_package().pads.emplace(u, Pad(u, it.value(), doc.r->get_pool())).first->second;
             apply_shift(x.placement.shift, cursor_pos_canvas);
             pads.push_back(&x);
             selection.emplace(u, ObjectType::PAD);
         }
-        doc.k->get_package()->apply_parameter_set({});
+        doc.k->get_package().apply_parameter_set({});
         if (max_name > 0) {
             std::sort(pads.begin(), pads.end(),
                       [](const Pad *a, const Pad *b) { return strcmp_natural(a->name, b->name) < 0; });
@@ -226,7 +226,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
             component_xlat.emplace(it.key(), u);
-            Component comp(u, it.value(), *doc.r->get_pool());
+            Component comp(u, it.value(), doc.r->get_pool());
             for (auto &it_conn : comp.connections) {
                 it_conn.second.net = &block->nets.at(net_xlat.at(it_conn.second.net.uuid));
             }
@@ -241,7 +241,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
             symbol_xlat.emplace(it.key(), u);
-            SchematicSymbol sym(u, it.value(), *doc.r->get_pool());
+            SchematicSymbol sym(u, it.value(), doc.r->get_pool());
             sym.component = &block->components.at(component_xlat.at(sym.component.uuid));
             sym.gate = &sym.component->entity->gates.at(sym.gate.uuid);
             auto x = &sheet->symbols.emplace(u, sym).first->second;
@@ -342,8 +342,8 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         const json &o = j["shapes"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
-            auto x = &doc.a->get_padstack()->shapes.emplace(u, Shape(u, it.value())).first->second;
-            x->placement.shift += shift;
+            auto &x = doc.a->get_padstack().shapes.emplace(u, Shape(u, it.value())).first->second;
+            x.placement.shift += shift;
             selection.emplace(u, ObjectType::SHAPE);
         }
     }
@@ -356,7 +356,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
                               .emplace(std::piecewise_construct, std::forward_as_tuple(u),
                                        std::forward_as_tuple(u, it.value()))
                               .first->second;
-            if (auto ps = doc.b->get_via_padstack_provider()->get_padstack(x->vpp_padstack.uuid)) {
+            if (auto ps = doc.b->get_via_padstack_provider().get_padstack(x->vpp_padstack.uuid)) {
                 x->vpp_padstack = ps;
                 x->padstack = *ps;
                 x->padstack.apply_parameter_set(x->parameter_set);
@@ -422,6 +422,19 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             }
         }
     }
+    if (j.count("decals") && doc.b) {
+        const json &o = j["decals"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            auto u = UUID::random();
+            auto brd = doc.b->get_board();
+            auto &x = brd->decals
+                              .emplace(std::piecewise_construct, std::forward_as_tuple(u),
+                                       std::forward_as_tuple(u, it.value(), doc.r->get_pool()))
+                              .first->second;
+            apply_shift(x.placement.shift, cursor_pos_canvas);
+            selection.emplace(u, ObjectType::BOARD_DECAL);
+        }
+    }
     if (selection.size() == 0) {
         imp->tool_bar_flash("Empty buffer");
         return ToolResponse::revert();
