@@ -12,6 +12,7 @@ public:
     ToolResponse begin(const ToolArgs &args) override;
     ToolResponse update(const ToolArgs &args) override;
     bool can_begin() override;
+    bool is_specific() override;
 
     class Settings : public ToolSettings {
     public:
@@ -54,6 +55,7 @@ protected:
 
 private:
     class Text *temp = 0;
+    class BoardPackage *pkg = nullptr;
     std::forward_list<Text *> texts_placed;
     Settings settings;
 };
