@@ -9,13 +9,13 @@ namespace horizon {
 
 class BoardHoleDialog : public Gtk::Dialog {
 public:
-    BoardHoleDialog(Gtk::Window *parent, std::set<class BoardHole *> &pads, class Pool *p, class Block *block);
+    BoardHoleDialog(Gtk::Window *parent, std::set<class BoardHole *> &pads, class IPool &p, class Block &block);
     bool valid = false;
 
 
 private:
-    class Pool *pool;
-    class Block *block;
+    class IPool &pool;
+    class Block &block;
     class MyParameterSetEditor *editor = nullptr;
     class PoolBrowserButton *padstack_button = nullptr;
     class NetButton *net_button = nullptr;
