@@ -9,23 +9,23 @@
 #include "common/text.hpp"
 #include "nlohmann/json_fwd.hpp"
 #include "util/uuid.hpp"
-#include "util/uuid_provider.hpp"
 #include <fstream>
 #include <map>
 #include <set>
 #include <vector>
+#include "util/file_version.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
 
-class Frame : public ObjectProvider, public LayerProvider, public UUIDProvider {
+class Frame : public ObjectProvider, public LayerProvider {
 public:
     Frame(const UUID &uu, const json &j);
     Frame(const UUID &uu);
     static Frame new_from_file(const std::string &filename);
     std::pair<Coordi, Coordi> get_bbox(bool all = false) const;
     Junction *get_junction(const UUID &uu) override;
-    UUID get_uuid() const override;
+    UUID get_uuid() const;
 
     json serialize() const;
 
@@ -44,6 +44,8 @@ public:
     int64_t width = 297_mm;
     int64_t height = 210_mm;
 
+    FileVersion version;
+
 private:
     void update_refs();
 };
