@@ -92,6 +92,14 @@ enum class ActionID {
     DISTRACTION_FREE,
     TOGGLE_SNAP_TO_TARGETS,
     PARTS_WINDOW,
+    PAN_UP,
+    PAN_DOWN,
+    PAN_LEFT,
+    PAN_RIGHT,
+    ZOOM_IN,
+    ZOOM_OUT,
+    CLICK_SELECT,
+    HIGHLIGHT_NET_CLASS,
 };
 
 using ActionToolID = std::pair<ActionID, ToolID>;
