@@ -395,7 +395,7 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
         {{ActionID::TOOL, ToolID::IMPORT_DXF},
          {"Import dxf", ActionGroup::EXPORT_IMPORT,
           ActionCatalogItem::AVAILABLE_IN_BOARD | ActionCatalogItem::AVAILABLE_IN_PACKAGE
-                  | ActionCatalogItem::AVAILABLE_IN_FRAME,
+                  | ActionCatalogItem::AVAILABLE_IN_FRAME | ActionCatalogItem::AVAILABLE_IN_DECAL,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::PLACE_JUNCTION},
@@ -418,6 +418,10 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
          {"Edit line rectangle", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::TOOL, ToolID::DRAW_LINE_CIRCLE},
+         {"Draw line circle", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::TOOL, ToolID::PLACE_PAD},
          {"Place pad", ActionGroup::PACKAGE, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
           ActionCatalogItem::FLAGS_DEFAULT}},
@@ -714,7 +718,8 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
                   | ActionCatalogItem::FLAGS_NO_PREFERENCES}},
 
         {{ActionID::TOOL, ToolID::IMPORT_KICAD_PACKAGE},
-         {"Import KiCad package", ActionGroup::EXPORT_IMPORT, ActionCatalogItem::AVAILABLE_IN_PACKAGE,
+         {"Import KiCad package", ActionGroup::EXPORT_IMPORT,
+          ActionCatalogItem::AVAILABLE_IN_PACKAGE | ActionCatalogItem::AVAILABLE_IN_DECAL,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::SMASH_SILKSCREEN_GRAPHICS},
@@ -838,6 +843,44 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
 
         {{ActionID::PARTS_WINDOW, ToolID::NONE},
          {"Parts window", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::ADD_TEXT},
+         {"Add text", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::PAN_LEFT, ToolID::NONE},
+         {"Pan left", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_NO_POPOVER | ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::PAN_RIGHT, ToolID::NONE},
+         {"Pan right", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_NO_POPOVER | ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::PAN_UP, ToolID::NONE},
+         {"Pan up", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_NO_POPOVER | ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::PAN_DOWN, ToolID::NONE},
+         {"Pan down", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_NO_POPOVER | ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::ZOOM_IN, ToolID::NONE},
+         {"Zoom in", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_NO_POPOVER | ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::ZOOM_OUT, ToolID::NONE},
+         {"Zoom out", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_NO_POPOVER | ActionCatalogItem::FLAGS_IN_TOOL}},
+
+        {{ActionID::TOOL, ToolID::PLACE_DECAL},
+         {"Place decal", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::CLICK_SELECT, ToolID::NONE},
+         {"Click select mode", ActionGroup::SELECTION, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::HIGHLIGHT_NET_CLASS, ToolID::NONE},
+         {"Highlight net class", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
+          ActionCatalogItem::FLAGS_SPECIFIC}},
 };
 
 const std::vector<std::pair<ActionGroup, std::string>> action_group_catalog = {
@@ -951,6 +994,14 @@ const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(SET_GRID_ORIGIN),
         ACTION_LUT_ITEM(TOGGLE_SNAP_TO_TARGETS),
         ACTION_LUT_ITEM(PARTS_WINDOW),
+        ACTION_LUT_ITEM(PAN_UP),
+        ACTION_LUT_ITEM(PAN_DOWN),
+        ACTION_LUT_ITEM(PAN_LEFT),
+        ACTION_LUT_ITEM(PAN_RIGHT),
+        ACTION_LUT_ITEM(ZOOM_IN),
+        ACTION_LUT_ITEM(ZOOM_OUT),
+        ACTION_LUT_ITEM(CLICK_SELECT),
+        ACTION_LUT_ITEM(HIGHLIGHT_NET_CLASS),
 };
 
 #define TOOL_LUT_ITEM(x)                                                                                               \
@@ -1097,5 +1148,8 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(ROUND_OFF_VERTEX),
         TOOL_LUT_ITEM(SWAP_GATES),
         TOOL_LUT_ITEM(PLACE_PICTURE),
+        TOOL_LUT_ITEM(DRAW_LINE_CIRCLE),
+        TOOL_LUT_ITEM(ADD_TEXT),
+        TOOL_LUT_ITEM(PLACE_DECAL),
 };
 } // namespace horizon
