@@ -19,12 +19,7 @@
 #include "grid_controller.hpp"
 #include "util/action_label.hpp"
 #include <optional>
-
-#ifdef G_OS_WIN32
-#undef DELETE
-#undef DUPLICATE
-#undef ERROR
-#endif
+#include "util/win32_undef.hpp"
 
 namespace horizon {
 
@@ -58,14 +53,19 @@ public:
 
     class SelectionFilterInfo {
     public:
+        enum class Flag {
+            DEFAULT,
+            HAS_OTHERS,
+            WORK_LAYER_ONLY_ENABLED,
+        };
         SelectionFilterInfo()
         {
         }
-        SelectionFilterInfo(const std::vector<int> &l, bool o) : layers(l), has_others(o)
+        SelectionFilterInfo(const std::vector<int> &l, Flag fl) : layers(l), flags(fl)
         {
         }
         std::vector<int> layers;
-        bool has_others = false;
+        Flag flags = Flag::DEFAULT;
     };
 
     virtual std::map<ObjectType, SelectionFilterInfo> get_selection_filter_info() const;
@@ -220,8 +220,11 @@ protected:
 
     class ActionButton &add_action_button(ActionToolID action);
     class ActionButtonMenu &add_action_button_menu(const char *icon_name);
+    class ActionButton &add_action_button_polygon();
+    class ActionButton &add_action_button_line();
 
     virtual ToolID get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef> &sel) const;
+    void force_end_tool();
 
 private:
     void fix_cursor_pos();
@@ -244,7 +247,8 @@ private:
     Gtk::MenuItem *create_context_menu_item(ActionToolID act);
 
     KeySequence keys_current;
-    bool keys_match(const KeySequence &keys) const;
+    enum class MatchResult { NONE, PREFIX, COMPLETE };
+    MatchResult keys_match(const KeySequence &keys) const;
     bool handle_action_key(GdkEventKey *ev);
     void handle_tool_action(const ActionConnection &conn);
     void handle_select_polygon(const ActionConnection &a);
@@ -266,8 +270,6 @@ private:
     std::map<ActionToolID, bool> action_sensitivity;
     type_signal_action_sensitive s_signal_action_sensitive;
 
-    GdkModifierType grid_fine_modifier = GDK_MOD1_MASK;
-
     bool property_panel_has_focus();
 
     sigc::connection initial_view_all_conn;
@@ -291,5 +293,20 @@ private:
     void tool_bar_clear_actions();
     InToolKeySequencesPreferences in_tool_key_sequeces_preferences;
     std::vector<ActionLabelInfo> in_tool_action_label_infos;
+
+    void show_preferences(std::optional<std::string> page = std::nullopt);
+
+    void init_search();
+    void init_key();
+    void init_action();
+
+    void handle_pan_action(const ActionConnection &c);
+    void handle_zoom_action(const ActionConnection &c);
+
+    std::string get_complete_display_name(const SelectableRef &sr);
+
+    void set_flip_view(bool flip);
+    void apply_arrow_keys();
+    void check_version();
 };
 } // namespace horizon
