@@ -140,33 +140,30 @@ void ImpSchematic::update_highlights()
 bool ImpSchematic::handle_broadcast(const json &j)
 {
     if (!ImpBase::handle_broadcast(j)) {
-        if (core_schematic.tool_is_active())
-            return true;
         std::string op = j.at("op");
         guint32 timestamp = j.value("time", 0);
         if (op == "place-part") {
+            force_end_tool();
             main_window->present(timestamp);
             auto data = std::make_unique<ToolAddPart::ToolDataAddPart>(j.at("part").get<std::string>());
             tool_begin(ToolID::ADD_PART, false, {}, std::move(data));
         }
-        else if (op == "assign-part") {
+        else if (op == "assign-part" && !core->tool_is_active()) {
             main_window->present(timestamp);
             auto data = std::make_unique<ToolAddPart::ToolDataAddPart>(j.at("part").get<std::string>());
             tool_begin(ToolID::ASSIGN_PART, false, {}, std::move(data));
         }
-        else if (op == "highlight" && cross_probing_enabled) {
-            if (!core_schematic.tool_is_active()) {
-                highlights.clear();
-                const json &o = j["objects"];
-                for (auto it = o.cbegin(); it != o.cend(); ++it) {
-                    auto type = static_cast<ObjectType>(it.value().at("type").get<int>());
-                    UUID uu(it.value().at("uuid").get<std::string>());
-                    highlights.emplace(type, uu);
-                }
-                update_highlights();
+        else if (op == "highlight" && cross_probing_enabled && !core->tool_is_active()) {
+            highlights.clear();
+            const json &o = j["objects"];
+            for (auto it = o.cbegin(); it != o.cend(); ++it) {
+                auto type = static_cast<ObjectType>(it.value().at("type").get<int>());
+                UUID uu(it.value().at("uuid").get<std::string>());
+                highlights.emplace(type, uu);
             }
+            update_highlights();
         }
-        else if (op == "backannotate" && !core_schematic.tool_is_active()) {
+        else if (op == "backannotate") {
             auto data = std::make_unique<ToolBackannotateConnectionLines::ToolDataBackannotate>();
             const json &o = j.at("connections");
             for (auto it = o.cbegin(); it != o.cend(); ++it) {
@@ -181,11 +178,13 @@ bool ImpSchematic::handle_broadcast(const json &j)
                     data->connections.push_back(item);
             }
             if (data->connections.size()) {
+                force_end_tool();
                 main_window->present(timestamp);
                 tool_begin(ToolID::BACKANNOTATE_CONNECTION_LINES, true, {}, std::move(data));
             }
         }
-        else if (op == "edit-meta" && !core_schematic.tool_is_active()) {
+        else if (op == "edit-meta") {
+            force_end_tool();
             main_window->present(timestamp);
             tool_begin(ToolID::EDIT_SCHEMATIC_PROPERTIES);
         }
@@ -410,34 +409,23 @@ void ImpSchematic::construct()
 
     connect_action(ActionID::HIGHLIGHT_NET, [this](const auto &a) {
         highlights.clear();
-        auto sheet = core_schematic.get_sheet();
         for (const auto &it : canvas->get_selection()) {
-            switch (it.type) {
-            case ObjectType::LINE_NET: {
-                auto &li = sheet->net_lines.at(it.uuid);
-                if (li.net) {
-                    highlights.emplace(ObjectType::NET, li.net->uuid);
+            if (auto uu = net_from_selectable(it)) {
+                highlights.emplace(ObjectType::NET, uu);
+            }
+        }
+        this->update_highlights();
+    });
+
+    connect_action(ActionID::HIGHLIGHT_NET_CLASS, [this](const auto &a) {
+        highlights.clear();
+        for (const auto &it : canvas->get_selection()) {
+            if (auto uu = net_from_selectable(it)) {
+                const auto &net_sel = core_schematic.get_block()->nets.at(uu);
+                for (const auto &[net_uu, net] : core_schematic.get_block()->nets) {
+                    if (net.net_class == net_sel.net_class)
+                        highlights.emplace(ObjectType::NET, net_uu);
                 }
-            } break;
-            case ObjectType::NET_LABEL: {
-                auto &la = sheet->net_labels.at(it.uuid);
-                if (la.junction->net) {
-                    highlights.emplace(ObjectType::NET, la.junction->net->uuid);
-                }
-            } break;
-            case ObjectType::POWER_SYMBOL: {
-                auto &sym = sheet->power_symbols.at(it.uuid);
-                if (sym.junction->net) {
-                    highlights.emplace(ObjectType::NET, sym.junction->net->uuid);
-                }
-            } break;
-            case ObjectType::JUNCTION: {
-                auto &ju = sheet->junctions.at(it.uuid);
-                if (ju.net) {
-                    highlights.emplace(ObjectType::NET, ju.net->uuid);
-                }
-            } break;
-            default:;
             }
         }
         this->update_highlights();
@@ -446,8 +434,8 @@ void ImpSchematic::construct()
     connect_action(ActionID::HIGHLIGHT_GROUP, sigc::mem_fun(*this, &ImpSchematic::handle_highlight_group_tag));
     connect_action(ActionID::HIGHLIGHT_TAG, sigc::mem_fun(*this, &ImpSchematic::handle_highlight_group_tag));
 
-    bom_export_window = BOMExportWindow::create(main_window, core_schematic.get_block(),
-                                                core_schematic.get_bom_export_settings(), *pool.get(), project_dir);
+    bom_export_window = BOMExportWindow::create(main_window, *core_schematic.get_block(),
+                                                *core_schematic.get_bom_export_settings(), *pool.get(), project_dir);
 
     connect_action(ActionID::BOM_EXPORT_WINDOW, [this](const auto &c) { bom_export_window->present(); });
     connect_action(ActionID::EXPORT_BOM, [this](const auto &c) {
@@ -459,8 +447,8 @@ void ImpSchematic::construct()
     core->signal_tool_changed().connect([this](ToolID t) { bom_export_window->set_can_export(t == ToolID::NONE); });
     core->signal_rebuilt().connect([this] { bom_export_window->update(); });
 
-    pdf_export_window = PDFExportWindow::create(main_window, dynamic_cast<IDocument *>(&core_schematic),
-                                                *core_schematic.get_pdf_export_settings(), project_dir);
+    pdf_export_window = PDFExportWindow::create(main_window, core_schematic, *core_schematic.get_pdf_export_settings(),
+                                                project_dir);
     pdf_export_window->signal_changed().connect([this] { core_schematic.set_needs_save(); });
     connect_action(ActionID::PDF_EXPORT_WINDOW, [this](const auto &c) { pdf_export_window->present(); });
     connect_action(ActionID::EXPORT_PDF, [this](const auto &c) {
@@ -542,11 +530,7 @@ void ImpSchematic::construct()
         b.add_action(make_action(ToolID::PLACE_BUS_RIPPER));
     }
     add_action_button(make_action(ToolID::PLACE_POWER_SYMBOL));
-    {
-        auto &b = add_action_button(make_action(ToolID::DRAW_LINE));
-        b.add_action(make_action(ToolID::DRAW_LINE_RECTANGLE));
-        b.add_action(make_action(ToolID::DRAW_ARC));
-    }
+    add_action_button_line();
     add_action_button(make_action(ToolID::PLACE_TEXT));
 
     if (!core_schematic.get_project_meta_loaded_from_block())
@@ -580,18 +564,20 @@ void ImpSchematic::update_action_sensitivity()
     set_action_sensitive(make_action(ActionID::MOVE_TO_OTHER_SHEET), sel.size() > 0);
     set_action_sensitive(make_action(ActionID::GO_TO_BOARD), sockets_connected);
 
-    set_action_sensitive(make_action(ActionID::HIGHLIGHT_NET), std::any_of(sel.begin(), sel.end(), [](const auto &x) {
-                             switch (x.type) {
-                             case ObjectType::LINE_NET:
-                             case ObjectType::NET_LABEL:
-                             case ObjectType::JUNCTION:
-                             case ObjectType::POWER_SYMBOL:
-                                 return true;
+    const bool can_highlight_net = std::any_of(sel.begin(), sel.end(), [](const auto &x) {
+        switch (x.type) {
+        case ObjectType::LINE_NET:
+        case ObjectType::NET_LABEL:
+        case ObjectType::JUNCTION:
+        case ObjectType::POWER_SYMBOL:
+            return true;
 
-                             default:
-                                 return false;
-                             }
-                         }));
+        default:
+            return false;
+        }
+    });
+    set_action_sensitive(make_action(ActionID::HIGHLIGHT_NET), can_highlight_net);
+    set_action_sensitive(make_action(ActionID::HIGHLIGHT_NET_CLASS), can_highlight_net);
     bool can_higlight_group = false;
     bool can_higlight_tag = false;
     if (sel.size() == 1 && (*sel.begin()).type == ObjectType::SCHEMATIC_SYMBOL) {
@@ -609,6 +595,39 @@ void ImpSchematic::update_action_sensitivity()
     ImpBase::update_action_sensitivity();
 }
 
+UUID ImpSchematic::net_from_selectable(const SelectableRef &sr)
+{
+    const auto &sheet = *core_schematic.get_sheet();
+    switch (sr.type) {
+    case ObjectType::LINE_NET: {
+        auto &li = sheet.net_lines.at(sr.uuid);
+        if (li.net) {
+            return li.net->uuid;
+        }
+    } break;
+    case ObjectType::NET_LABEL: {
+        auto &la = sheet.net_labels.at(sr.uuid);
+        if (la.junction->net) {
+            return la.junction->net->uuid;
+        }
+    } break;
+    case ObjectType::POWER_SYMBOL: {
+        auto &sym = sheet.power_symbols.at(sr.uuid);
+        if (sym.junction->net) {
+            return sym.junction->net->uuid;
+        }
+    } break;
+    case ObjectType::JUNCTION: {
+        auto &ju = sheet.junctions.at(sr.uuid);
+        if (ju.net) {
+            return ju.net->uuid;
+        }
+    } break;
+    default:;
+    }
+    return UUID();
+}
+
 std::string ImpSchematic::get_hud_text(std::set<SelectableRef> &sel)
 {
     std::string s;
