@@ -9,7 +9,7 @@
 
 namespace horizon {
 ImpSymbol::ImpSymbol(const std::string &symbol_filename, const std::string &pool_path)
-    : ImpBase(pool_path), core_symbol(symbol_filename, *pool), searcher(core_symbol)
+    : ImpBase(pool_path), core_symbol(symbol_filename, *pool), symbol(core_symbol.get_symbol()), searcher(core_symbol)
 {
     core = &core_symbol;
     core_symbol.signal_tool_changed().connect(sigc::mem_fun(*this, &ImpBase::handle_tool_change));
@@ -24,7 +24,7 @@ void ImpSymbol::canvas_update()
 
 void ImpSymbol::update_monitor()
 {
-    set_monitor_items({{ObjectType::UNIT, core_symbol.get_symbol()->unit->uuid}});
+    set_monitor_items({{ObjectType::UNIT, symbol.unit->uuid}});
 }
 
 void ImpSymbol::apply_preferences()
@@ -39,7 +39,7 @@ void ImpSymbol::construct()
     state_store = std::make_unique<WindowStateStore>(main_window, "imp-symbol");
 
     symbol_preview_window = new SymbolPreviewWindow(main_window);
-    symbol_preview_window->set_text_placements(core_symbol.get_symbol()->text_placements);
+    symbol_preview_window->set_text_placements(symbol.text_placements);
     symbol_preview_window->signal_changed().connect([this] { core_symbol.set_needs_save(); });
     core_symbol.signal_tool_changed().connect(
             [this](ToolID tool_id) { symbol_preview_window->set_can_load(tool_id == ToolID::NONE); });
@@ -47,7 +47,7 @@ void ImpSymbol::construct()
         if (core_symbol.tool_is_active())
             return;
         auto pl = symbol_preview_window->get_text_placements();
-        for (auto &it : core_symbol.get_symbol()->texts) {
+        for (auto &it : symbol.texts) {
             auto key = std::make_tuple(angle, mirror, it.first);
             if (pl.count(key))
                 it.second.placement = pl.at(key);
@@ -88,14 +88,14 @@ void ImpSymbol::construct()
     header_button->signal_closed().connect(sigc::mem_fun(*this, &ImpSymbol::update_header));
 
     name_entry = header_button->add_entry("Name");
-    name_entry->set_text(core_symbol.get_symbol()->name);
-    name_entry->set_width_chars(std::min<int>(core_symbol.get_symbol()->name.size(), 20));
+    name_entry->set_text(symbol.name);
+    name_entry->set_width_chars(std::min<int>(symbol.name.size(), 20));
     name_entry->signal_changed().connect([this] { core_symbol.set_needs_save(); });
     name_entry->signal_activate().connect(sigc::mem_fun(*this, &ImpSymbol::update_header));
 
     {
         auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 10));
-        unit_label = Gtk::manage(new Gtk::Label(core_symbol.get_symbol()->unit->name));
+        unit_label = Gtk::manage(new Gtk::Label(symbol.unit->name));
         unit_label->set_xalign(0);
         unit_label->set_selectable(true);
         box->pack_start(*unit_label, true, true, 0);
@@ -107,7 +107,7 @@ void ImpSymbol::construct()
     }
 
     can_expand_switch = Gtk::manage(new Gtk::Switch);
-    can_expand_switch->set_active(core_symbol.get_symbol()->can_expand);
+    can_expand_switch->set_active(symbol.can_expand);
     can_expand_switch->set_halign(Gtk::ALIGN_START);
     can_expand_switch->property_active().signal_changed().connect([this] { core_symbol.set_needs_save(); });
 
@@ -151,20 +151,18 @@ void ImpSymbol::construct()
                 [this, box](ToolID tool_id) { box->set_sensitive(tool_id == ToolID::NONE); });
     }
 
-    core->signal_rebuilt().connect([this] { unit_label->set_text(core_symbol.get_symbol()->unit->name); });
+    core->signal_rebuilt().connect([this] { unit_label->set_text(symbol.unit->name); });
 
     core->signal_save().connect([this] {
-        auto sym = core_symbol.get_symbol();
-        sym->name = name_entry->get_text();
-        sym->text_placements = symbol_preview_window->get_text_placements();
-        sym->can_expand = can_expand_switch->get_active();
+        symbol.name = name_entry->get_text();
+        symbol.text_placements = symbol_preview_window->get_text_placements();
+        symbol.can_expand = can_expand_switch->get_active();
     });
     grid_controller->disable();
     update_monitor();
 
-    connect_action(ActionID::EDIT_UNIT, [this](const auto &a) {
-        this->edit_pool_item(ObjectType::UNIT, core_symbol.get_symbol()->unit->uuid);
-    });
+    connect_action(ActionID::EDIT_UNIT,
+                   [this](const auto &a) { this->edit_pool_item(ObjectType::UNIT, symbol.unit->uuid); });
     set_action_sensitive(make_action(ActionID::EDIT_UNIT), sockets_connected);
 
     hamburger_menu->append("Change unit", "win.change_unit");
@@ -190,6 +188,7 @@ void ImpSymbol::construct()
     add_action_button(make_action(ToolID::PLACE_REFDES_AND_VALUE));
     {
         auto &x = add_action_button(make_action(ToolID::DRAW_LINE));
+        x.add_action(make_action(ToolID::DRAW_LINE_CIRCLE));
         x.add_action(make_action(ToolID::DRAW_ARC));
     }
     add_action_button(make_action(ToolID::PLACE_TEXT));
@@ -204,9 +203,9 @@ void ImpSymbol::update_header()
 
 void ImpSymbol::update_unplaced()
 {
-    const auto &pins_from_symbol = core_symbol.get_symbol()->pins;
+    const auto &pins_from_symbol = symbol.pins;
     std::map<UUIDPath<2>, std::string> unplaced;
-    for (const auto &it : core_symbol.get_symbol()->unit->pins) {
+    for (const auto &it : symbol.unit->pins) {
         if (pins_from_symbol.count(it.first) == 0) {
             unplaced.emplace(it.first, it.second.primary_name);
         }
@@ -219,7 +218,7 @@ void ImpSymbol::update_bbox_annotation()
     if (!bbox_annotation)
         return;
     Coordi a;
-    for (const auto &it : core_symbol.get_symbol()->pins) {
+    for (const auto &it : symbol.pins) {
         const auto &p = it.second.position;
         a = Coordi::max(a, {std::abs(p.x), std::abs(p.y)});
     }
@@ -238,7 +237,7 @@ void ImpSymbol::handle_selection_cross_probe()
         return;
     json j;
     j["op"] = "symbol-select";
-    j["unit"] = (std::string)core_symbol.get_symbol()->unit->uuid;
+    j["unit"] = (std::string)symbol.unit->uuid;
     auto pins = json::array();
     for (const auto &it : canvas->get_selection()) {
         if (it.type == ObjectType::SYMBOL_PIN) {
