@@ -9,56 +9,61 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     : Gtk::ApplicationWindow(cobject), builder(x)
 {
 
-    x->get_widget("gl_container", gl_container);
-    x->get_widget("tool_hint_label", tool_hint_label);
-    x->get_widget("left_panel", left_panel);
-    x->get_widget("grid_box_square", grid_box_square);
-    x->get_widget("grid_box_rect", grid_box_rect);
-    x->get_widget("grid_box_stack", grid_box_stack);
-    x->get_widget("grid_mul_label", grid_mul_label);
-    x->get_widget("cursor_label", cursor_label);
-    x->get_widget("selection_mode_label", selection_mode_label);
-    x->get_widget("property_viewport", property_viewport);
-    x->get_widget("tool_bar", tool_bar);
-    x->get_widget("tool_bar_label", tool_bar_name_label);
-    x->get_widget("tool_bar_stack", tool_bar_stack);
-    x->get_widget("tool_bar_tip", tool_bar_tip_label);
-    x->get_widget("tool_bar_flash", tool_bar_flash_label);
-    x->get_widget("selection_label", selection_label);
-    x->get_widget("header", header);
-    x->get_widget("property_scrolled_window", property_scrolled_window);
-    x->get_widget("property_throttled_revealer", property_throttled_revealer);
-    x->get_widget("hud", hud);
-    x->get_widget("hud_label", hud_label);
-    x->get_widget("nonmodal_rev", nonmodal_rev);
-    x->get_widget("nonmodal_button", nonmodal_button);
-    x->get_widget("nonmodal_close_button", nonmodal_close_button);
-    x->get_widget("nonmodal_label", nonmodal_label);
-    x->get_widget("nonmodal_label2", nonmodal_label2);
-    x->get_widget("view_hints_label", view_hints_label);
+    GET_WIDGET(gl_container);
+    GET_WIDGET(tool_hint_label);
+    GET_WIDGET(left_panel);
+    GET_WIDGET(grid_box_square);
+    GET_WIDGET(grid_box_rect);
+    GET_WIDGET(grid_box_stack);
+    GET_WIDGET(grid_mul_label);
+    GET_WIDGET(cursor_label);
+    GET_WIDGET(selection_mode_label);
+    GET_WIDGET(property_viewport);
+    GET_WIDGET(tool_bar);
+    GET_WIDGET(tool_bar_name_label);
+    GET_WIDGET(tool_bar_stack);
+    GET_WIDGET(tool_bar_tip_label);
+    GET_WIDGET(tool_bar_flash_label);
+    GET_WIDGET(selection_label);
+    GET_WIDGET(header);
+    GET_WIDGET(property_scrolled_window);
+    GET_WIDGET(property_throttled_revealer);
+    GET_WIDGET(hud);
+    GET_WIDGET(hud_label);
+    GET_WIDGET(nonmodal_rev);
+    GET_WIDGET(nonmodal_button);
+    GET_WIDGET(nonmodal_close_button);
+    GET_WIDGET(nonmodal_label);
+    GET_WIDGET(nonmodal_label2);
+    GET_WIDGET(view_hints_label);
 
-    x->get_widget("search_revealer", search_revealer);
-    x->get_widget("search_entry", search_entry);
-    x->get_widget("search_previous_button", search_previous_button);
-    x->get_widget("search_next_button", search_next_button);
-    x->get_widget("search_status_label", search_status_label);
-    x->get_widget("search_types_box", search_types_box);
-    x->get_widget("search_expander", search_expander);
-    x->get_widget("search_exact_cb", search_exact_cb);
-    x->get_widget("action_bar_revealer", action_bar_revealer);
-    x->get_widget("action_bar_box", action_bar_box);
-    x->get_widget("view_options_button", view_options_button);
+    GET_WIDGET(search_revealer);
+    GET_WIDGET(search_entry);
+    GET_WIDGET(search_previous_button);
+    GET_WIDGET(search_next_button);
+    GET_WIDGET(search_status_label);
+    GET_WIDGET(search_types_box);
+    GET_WIDGET(search_expander);
+    GET_WIDGET(search_exact_cb);
+    GET_WIDGET(action_bar_revealer);
+    GET_WIDGET(action_bar_box);
+    GET_WIDGET(view_options_button);
     search_revealer->set_reveal_child(false);
 
-    x->get_widget("grid_options_button", grid_options_button);
-    x->get_widget("grid_options_revealer", grid_options_revealer);
-    x->get_widget("grid_square_button", grid_square_button);
-    x->get_widget("grid_rect_button", grid_rect_button);
-    x->get_widget("grid_grid", grid_grid);
-    x->get_widget("grid_reset_origin_button", grid_reset_origin_button);
+    GET_WIDGET(grid_options_button);
+    GET_WIDGET(grid_options_revealer);
+    GET_WIDGET(grid_square_button);
+    GET_WIDGET(grid_rect_button);
+    GET_WIDGET(grid_grid);
+    GET_WIDGET(grid_reset_origin_button);
     GET_WIDGET(tool_bar_action_tip_label);
     GET_WIDGET(tool_bar_actions_box);
 
+    GET_WIDGET(version_info_bar);
+    GET_WIDGET(version_label);
+
+    set_version_info("");
+
     grid_options_button->signal_clicked().connect([this] {
         const auto a = grid_options_button->get_active();
         grid_options_revealer->set_reveal_child(a);
@@ -246,6 +251,17 @@ void MainWindow::tool_bar_append_action(Gtk::Widget &w)
     tool_bar_actions_box->pack_start(w, false, false, 0);
 }
 
+void MainWindow::set_version_info(const std::string &s)
+{
+    if (s.size()) {
+        info_bar_show(version_info_bar);
+        version_label->set_markup(s);
+    }
+    else {
+        info_bar_hide(version_info_bar);
+    }
+}
+
 MainWindow *MainWindow::create()
 {
     MainWindow *w;
