@@ -8,7 +8,7 @@
 namespace horizon {
 class RuleEditor : public Gtk::Box {
 public:
-    RuleEditor(Rule *r, class IDocument *c);
+    RuleEditor(Rule &r, class IDocument &c);
     virtual void populate();
     typedef sigc::signal<void> type_signal_updated;
     type_signal_updated signal_updated()
@@ -20,11 +20,11 @@ private:
     Gtk::CheckButton *enable_cb = nullptr;
 
 protected:
-    Rule *rule;
-    class IDocument *core = nullptr;
+    Rule &rule;
+    class IDocument &core;
     Glib::RefPtr<Gtk::Builder> builder;
     class SpinButtonDim *create_spinbutton(const char *box);
-    class RuleMatchEditor *create_rule_match_editor(const char *box, class RuleMatch *match);
+    class RuleMatchEditor *create_rule_match_editor(const char *box, class RuleMatch &match);
     type_signal_updated s_signal_updated;
 };
 } // namespace horizon
