@@ -14,12 +14,14 @@
 #include "rule_editor_via.hpp"
 #include "rule_editor_clearance_copper_keepout.hpp"
 #include "rule_editor_layer_pair.hpp"
+#include "rule_editor_clearance_same_net.hpp"
 #include "rules/cache.hpp"
 #include "rules/rule_descr.hpp"
 #include "rules/rules.hpp"
 #include "rules/rules_with_core.hpp"
 #include "util/util.hpp"
 #include "core/core.hpp"
+#include "util/gtk_util.hpp"
 
 #include <thread>
 
@@ -72,35 +74,43 @@ private:
     Gtk::Label *label_text = nullptr;
 };
 
-RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, CanvasGL *ca, Rules *ru,
-                         class Core *c)
-    : Gtk::Window(cobject), canvas(ca), rules(ru), core(c),
-      state_store(this, "imp-rules-" + object_type_lut.lookup_reverse(core->get_object_type()))
+RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, CanvasGL &ca, class Core &c)
+    : Gtk::Window(cobject), canvas(ca), core(c), rules(*core.get_rules()),
+      state_store(this, "imp-rules-" + object_type_lut.lookup_reverse(core.get_object_type()))
 {
-    x->get_widget("lb_rules", lb_rules);
-    x->get_widget("lb_multi", lb_multi);
-    x->get_widget("rev_multi", rev_multi);
-    x->get_widget("button_rule_instance_add", button_rule_instance_add);
-    x->get_widget("button_rule_instance_remove", button_rule_instance_remove);
-    x->get_widget("button_rule_instance_move_up", button_rule_instance_move_up);
-    x->get_widget("button_rule_instance_move_down", button_rule_instance_move_down);
-    x->get_widget("rule_editor_box", rule_editor_box);
-    x->get_widget("results_tv", check_result_treeview);
-    x->get_widget("run_button", run_button);
-    x->get_widget("apply_button", apply_button);
-    x->get_widget("stack", stack);
-    x->get_widget("stack_switcher", stack_switcher);
-    x->get_widget("rev_warn", rev_warn);
+    GET_WIDGET(lb_rules);
+    GET_WIDGET(lb_multi);
+    GET_WIDGET(rev_multi);
+    GET_WIDGET(button_rule_instance_add);
+    GET_WIDGET(button_rule_instance_remove);
+    GET_WIDGET(button_rule_instance_move_up);
+    GET_WIDGET(button_rule_instance_move_down);
+    GET_WIDGET(rule_editor_box);
+    GET_WIDGET(check_result_treeview);
+    GET_WIDGET(run_button);
+    GET_WIDGET(apply_button);
+    GET_WIDGET(stack);
+    GET_WIDGET(stack_switcher);
+    GET_WIDGET(rev_warn);
     sg_order = Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL);
 
     lb_rules->signal_row_selected().connect(
             [this](Gtk::ListBoxRow *row) { rule_selected(dynamic_cast<RuleLabel *>(row->get_child())->id); });
-    for (const auto id : rules->get_rule_ids()) {
+    for (const auto id : rules.get_rule_ids()) {
         const auto &desc = rule_descriptions.at(id);
         auto la = Gtk::manage(new RuleLabel(sg_order, desc.name, id));
         la->show();
         lb_rules->append(*la);
     }
+
+    {
+        const auto ids = rules.get_rule_ids();
+        const bool can_apply =
+                std::any_of(ids.begin(), ids.end(), [](const auto id) { return rule_descriptions.at(id).can_apply; });
+        if (!can_apply)
+            apply_button->hide();
+    }
+
     lb_multi->signal_selected_rows_changed().connect([this] {
         bool selected = lb_multi->get_selected_row() != nullptr;
         button_rule_instance_remove->set_sensitive(selected && enabled);
@@ -126,7 +136,7 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
         auto row = lb_multi->get_selected_row();
         if (row) {
             auto la = dynamic_cast<RuleLabel *>(row->get_child());
-            rules->remove_rule(la->id, la->uuid);
+            rules.remove_rule(la->id, la->uuid);
             update_rule_instances(la->id);
             update_warning();
             s_signal_changed.emit();
@@ -134,7 +144,7 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
     });
 
     button_rule_instance_add->signal_clicked().connect([this] {
-        rules->add_rule(rule_current);
+        rules.add_rule(rule_current);
         update_rule_instances(rule_current);
         lb_multi->select_row(*lb_multi->get_row_at_index(0));
         update_warning();
@@ -145,7 +155,7 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
         auto row = lb_multi->get_selected_row();
         if (row) {
             auto la = dynamic_cast<RuleLabel *>(row->get_child());
-            rules->move_rule(la->id, la->uuid, -1);
+            rules.move_rule(la->id, la->uuid, -1);
             update_rule_instance_labels();
             update_warning();
             s_signal_changed.emit();
@@ -155,7 +165,7 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
         auto row = lb_multi->get_selected_row();
         if (row) {
             auto la = dynamic_cast<RuleLabel *>(row->get_child());
-            rules->move_rule(la->id, la->uuid, 1);
+            rules.move_rule(la->id, la->uuid, 1);
             update_rule_instance_labels();
             update_warning();
             s_signal_changed.emit();
@@ -228,14 +238,14 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
         check_result_treeview->append_column(*tvc);
     }
 
-    annotation = canvas->create_annotation();
+    annotation = canvas.create_annotation();
 
     signal_show().connect([this] {
-        canvas->markers.set_domain_visible(MarkerDomain::CHECK, true);
+        canvas.markers.set_domain_visible(MarkerDomain::CHECK, true);
         annotation->set_visible(true);
     });
     signal_hide().connect([this] {
-        canvas->markers.set_domain_visible(MarkerDomain::CHECK, false);
+        canvas.markers.set_domain_visible(MarkerDomain::CHECK, false);
         annotation->set_visible(false);
     });
 
@@ -252,7 +262,7 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
 
     dispatcher.connect([this] {
         std::lock_guard<std::mutex> guard(run_store_mutex);
-        auto &dom = canvas->markers.get_domain(MarkerDomain::CHECK);
+        auto &dom = canvas.markers.get_domain(MarkerDomain::CHECK);
         for (const auto &it : run_store) {
             if (it.second.result.level != RulesCheckErrorLevel::NOT_RUN) { // has completed
                 it.second.row[tree_columns.result] = it.second.result.level;
@@ -285,7 +295,7 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
                 it.second.row[tree_columns.status] = it.second.status;
             }
         }
-        canvas->update_markers();
+        canvas.update_markers();
         check_result_treeview->expand_all();
         map_erase_if(run_store, [](auto &a) { return a.second.result.level != RulesCheckErrorLevel::NOT_RUN; });
         if (run_store.size() == 0) {
@@ -303,12 +313,12 @@ RulesWindow::RulesWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builde
 
 void RulesWindow::apply_rules()
 {
-    if (core->tool_is_active())
+    if (core.tool_is_active())
         return;
-    for (auto &rule : rules->get_rule_ids()) {
+    for (auto &rule : rules.get_rule_ids()) {
         rules_apply(rules, rule, core);
     }
-    core->rebuild();
+    core.rebuild();
     s_signal_canvas_update.emit();
     s_signal_changed.emit();
 }
@@ -352,9 +362,9 @@ void RulesWindow::check_thread(RuleID id)
 void RulesWindow::run_checks()
 {
     check_result_store->clear();
-    auto &dom = canvas->markers.get_domain(MarkerDomain::CHECK);
+    auto &dom = canvas.markers.get_domain(MarkerDomain::CHECK);
     dom.clear();
-    cache.reset(new RulesCheckCache(core));
+    cache.reset(new RulesCheckCache(&core));
     annotation->clear();
     run_store.clear();
     pulse_connection = Glib::signal_timeout().connect(sigc::track_obj(
@@ -367,7 +377,7 @@ void RulesWindow::run_checks()
                                                               },
                                                               *this),
                                                       750 / 12);
-    for (auto rule_id : rules->get_rule_ids()) {
+    for (auto rule_id : rules.get_rule_ids()) {
         if (rule_descriptions.at(rule_id).can_check) {
             Gtk::TreeModel::iterator iter = check_result_store->append();
             Gtk::TreeModel::Row row = *iter;
@@ -400,17 +410,17 @@ void RulesWindow::rule_selected(RuleID id)
         update_warning();
     }
     else {
-        auto rule = rules->get_rule(id);
-        show_editor(create_editor(rule));
+        auto rule = rules.get_rule(id);
+        show_editor(create_editor(*rule));
     }
 }
 
 void RulesWindow::rule_instance_selected(RuleID id, const UUID &uu)
 {
-    auto rule = rules->get_rule(id, uu);
+    auto rule = rules.get_rule(id, uu);
     if (rule == nullptr)
         return;
-    show_editor(create_editor(rule));
+    show_editor(create_editor(*rule));
 }
 
 void RulesWindow::show_editor(RuleEditor *e)
@@ -433,16 +443,17 @@ void RulesWindow::show_editor(RuleEditor *e)
     });
 }
 
-RuleEditor *RulesWindow::create_editor(Rule *r)
+RuleEditor *RulesWindow::create_editor(Rule &r)
 {
     RuleEditor *e = nullptr;
-    switch (r->id) {
+    switch (r.id) {
     case RuleID::HOLE_SIZE:
         e = new RuleEditorHoleSize(r, core);
         break;
 
     case RuleID::CLEARANCE_SILKSCREEN_EXPOSED_COPPER:
     case RuleID::PARAMETERS:
+    case RuleID::CLEARANCE_PACKAGE:
         e = new RuleEditorClearanceSilkscreenExposedCopper(r, core);
         break;
 
@@ -476,6 +487,7 @@ RuleEditor *RulesWindow::create_editor(Rule *r)
 
     case RuleID::PACKAGE_CHECKS:
     case RuleID::PREFLIGHT_CHECKS:
+    case RuleID::SYMBOL_CHECKS:
         e = new RuleEditorPackageChecks(r, core);
         break;
 
@@ -487,6 +499,10 @@ RuleEditor *RulesWindow::create_editor(Rule *r)
         e = new RuleEditorLayerPair(r, core);
         break;
 
+    case RuleID::CLEARANCE_SAME_NET:
+        e = new RuleEditorClearanceSameNet(r, core);
+        break;
+
     default:
         e = new RuleEditor(r, core);
     }
@@ -496,12 +512,12 @@ RuleEditor *RulesWindow::create_editor(Rule *r)
 
 Block *RulesWindow::get_block()
 {
-    return core->get_block();
+    return core.get_block();
 }
 
 void RulesWindow::update_rule_instances(RuleID id)
 {
-    auto inst = rules->get_rules(id);
+    auto inst = rules.get_rules(id);
     auto row = lb_multi->get_selected_row();
     UUID uu;
     if (row) {
@@ -531,7 +547,7 @@ void RulesWindow::update_rule_instance_labels()
 {
     for (auto ch : lb_multi->get_children()) {
         auto la = dynamic_cast<RuleLabel *>(dynamic_cast<Gtk::ListBoxRow *>(ch)->get_child());
-        auto rule = rules->get_rule(la->id, la->uuid);
+        auto rule = rules.get_rule(la->id, la->uuid);
         la->set_text(rule->get_brief(get_block()));
         la->set_sensitive(rule->enabled);
         la->set_order(rule->get_order());
@@ -545,7 +561,7 @@ void RulesWindow::update_rules_enabled()
         auto la = dynamic_cast<RuleLabel *>(dynamic_cast<Gtk::ListBoxRow *>(ch)->get_child());
         auto is_multi = rule_descriptions.at(la->id).multi;
         if (!is_multi) {
-            auto r = rules->get_rule(la->id);
+            auto r = rules.get_rule(la->id);
             la->set_sensitive(r->enabled);
         }
     }
@@ -571,7 +587,7 @@ void RulesWindow::update_warning()
         return;
     }
 
-    auto sorted = rules->get_rules_sorted(rule_current);
+    auto sorted = rules.get_rules_sorted(rule_current);
     if (sorted.size() == 0) {
         rev_warn->set_reveal_child(true);
     }
@@ -581,12 +597,12 @@ void RulesWindow::update_warning()
     }
 }
 
-RulesWindow *RulesWindow::create(Gtk::Window *p, CanvasGL *ca, Rules *ru, Core *c)
+RulesWindow *RulesWindow::create(Gtk::Window *p, CanvasGL &ca, Core &c)
 {
     RulesWindow *w;
     Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create();
     x->add_from_resource("/org/horizon-eda/horizon/imp/rules/rules_window.ui");
-    x->get_widget_derived("window", w, ca, ru, c);
+    x->get_widget_derived("window", w, ca, c);
     w->set_transient_for(*p);
     return w;
 }
