@@ -32,7 +32,7 @@ LocationEntry *PartWizard::pack_location_entry(const Glib::RefPtr<Gtk::Builder>
 }
 
 PartWizard::PartWizard(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, const UUID &pkg_uuid,
-                       const std::string &bp, class Pool *po, PoolProjectManagerAppWindow *aw)
+                       const std::string &bp, class Pool &po, PoolProjectManagerAppWindow *aw)
     : Gtk::Window(cobject), pool_base_path(bp), pool(po), part(UUID::random()), entity(UUID::random()), appwin(aw),
       state_store(this, "part-wizard")
 {
@@ -57,7 +57,7 @@ PartWizard::PartWizard(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     {
         Gtk::Box *tag_box;
         x->get_widget("entity_tags", tag_box);
-        entity_tags_entry = Gtk::manage(new TagEntry(*pool, ObjectType::ENTITY, true));
+        entity_tags_entry = Gtk::manage(new TagEntry(pool, ObjectType::ENTITY, true));
         entity_tags_entry->show();
         tag_box->pack_start(*entity_tags_entry, true, true, 0);
     }
@@ -70,7 +70,7 @@ PartWizard::PartWizard(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     {
         Gtk::Box *tag_box;
         x->get_widget("part_tags", tag_box);
-        part_tags_entry = Gtk::manage(new TagEntry(*pool, ObjectType::PART, true));
+        part_tags_entry = Gtk::manage(new TagEntry(pool, ObjectType::PART, true));
         part_tags_entry->show();
         tag_box->pack_start(*part_tags_entry, true, true, 0);
     }
@@ -145,7 +145,7 @@ PartWizard::PartWizard(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
         browser_package->show();
         page_package->add1(*browser_package);
 
-        auto preview = Gtk::manage(new PreviewCanvas(*pool, true));
+        auto preview = Gtk::manage(new PreviewCanvas(pool, true));
         button_select->set_sensitive(false);
         browser_package->signal_selected().connect([this, preview] {
             preview->load(ObjectType::PACKAGE, browser_package->get_selected());
@@ -269,7 +269,7 @@ void PartWizard::handle_select()
 {
     auto p = browser_package->get_selected();
     if (p) {
-        set_pkg(pool->get_package(p));
+        set_pkg(pool.get_package(p));
         set_mode(Mode::ASSIGN);
     }
 }
@@ -403,9 +403,9 @@ void PartWizard::finish()
             auto unit_filename = ed->unit_location_entry->get_filename();
             save_json_to_file(unit_filename, ed->gate->unit->serialize());
 
-            auto symbol_filename_src = pool->get_tmp_filename(ObjectType::SYMBOL, symbols.at(ed->gate->unit->uuid));
+            auto symbol_filename_src = pool.get_tmp_filename(ObjectType::SYMBOL, symbols.at(ed->gate->unit->uuid));
             auto symbol_filename_dest = ed->symbol_location_entry->get_filename();
-            auto sym = Symbol::new_from_file(symbol_filename_src, *pool);
+            auto sym = Symbol::new_from_file(symbol_filename_src, pool);
             sym.name = ed->symbol_name_entry->get_text();
             save_json_to_file(symbol_filename_dest, sym.serialize());
         }
@@ -424,6 +424,7 @@ void PartWizard::handle_link()
         editors.push_back(ed);
     }
     link_pads(editors);
+    update_pin_warnings();
 }
 
 void PartWizard::link_pads(const std::deque<PadEditor *> &editors)
@@ -691,9 +692,9 @@ void PartWizard::prepare_edit()
             ed->show_all();
             ed->edit_symbol_button->signal_clicked().connect([this, ed] {
                 auto symbol_uuid = symbols.at(ed->gate->unit->uuid);
-                auto symbol_filename = pool->get_tmp_filename(ObjectType::SYMBOL, symbol_uuid);
+                auto symbol_filename = pool.get_tmp_filename(ObjectType::SYMBOL, symbol_uuid);
                 {
-                    auto sym = Symbol::new_from_file(symbol_filename, *pool);
+                    auto sym = Symbol::new_from_file(symbol_filename, pool);
                     sym.name = ed->symbol_name_entry->get_text();
                     save_json_to_file(symbol_filename, sym.serialize());
                 }
@@ -705,7 +706,7 @@ void PartWizard::prepare_edit()
                     processes.erase(symbol_filename);
                     symbols_open.erase(symbol_uuid);
                     {
-                        auto sym = Symbol::new_from_file(symbol_filename, *pool);
+                        auto sym = Symbol::new_from_file(symbol_filename, pool);
                         ed->symbol_name_entry->set_text(sym.name);
                     }
                     update_can_finish();
@@ -824,13 +825,13 @@ void PartWizard::update_part()
             std::cout << "del sym" << std::endl;
 
             {
-                auto unit_filename = pool->get_tmp_filename(ObjectType::UNIT, uu);
+                auto unit_filename = pool.get_tmp_filename(ObjectType::UNIT, uu);
                 auto fi = Gio::File::create_for_path(unit_filename);
                 fi->remove();
             }
 
             {
-                auto sym_filename = pool->get_tmp_filename(ObjectType::SYMBOL, symbols.at(uu));
+                auto sym_filename = pool.get_tmp_filename(ObjectType::SYMBOL, symbols.at(uu));
                 auto fi = Gio::File::create_for_path(sym_filename);
                 fi->remove();
                 symbols.erase(uu);
@@ -851,11 +852,11 @@ void PartWizard::update_part()
             Symbol sym(UUID::random());
             sym.unit = &it.second;
             sym.name = "edit me";
-            auto filename = pool->get_tmp_filename(ObjectType::SYMBOL, sym.uuid);
+            auto filename = pool.get_tmp_filename(ObjectType::SYMBOL, sym.uuid);
             save_json_to_file(filename, sym.serialize());
             symbols.emplace(sym.unit->uuid, sym.uuid);
         }
-        auto filename = pool->get_tmp_filename(ObjectType::UNIT, it.second.uuid);
+        auto filename = pool.get_tmp_filename(ObjectType::UNIT, it.second.uuid);
         save_json_to_file(filename, it.second.serialize());
     }
 }
@@ -864,8 +865,8 @@ void PartWizard::update_symbol_pins_mapped()
 {
     for (const auto &it : symbols) {
         const auto &unit_uu = it.first;
-        auto symbol_filename = pool->get_tmp_filename(ObjectType::SYMBOL, it.second);
-        auto sym = Symbol::new_from_file(symbol_filename, *pool);
+        auto symbol_filename = pool.get_tmp_filename(ObjectType::SYMBOL, it.second);
+        auto sym = Symbol::new_from_file(symbol_filename, pool);
         auto n_pins = sym.pins.size();
         symbol_pins_mapped[unit_uu] = n_pins;
     }
@@ -912,7 +913,7 @@ void PartWizard::autofill()
 
 void PartWizard::reload()
 {
-    part.package = pool->get_package(part.package.uuid);
+    part.package = pool.get_package(part.package.uuid);
 }
 
 void PartWizard::update_can_finish()
@@ -938,7 +939,7 @@ void PartWizard::update_can_finish()
     check_entry_not_empty(part_mpn_entry, "MPN is empty", &mpn_valid);
     check_entry_not_empty(entity_name_entry, "Entity name is empty");
     check_entry_not_empty(entity_prefix_entry, "Entity prefix is empty");
-    valid = valid && part_location_entry->check_ends_json(&part_filename_valid);
+    valid = part_location_entry->check_ends_json(&part_filename_valid) && valid;
 
     std::set<std::string> symbol_filenames;
     std::set<std::string> unit_filenames;
@@ -956,8 +957,8 @@ void PartWizard::update_can_finish()
             entry_set_warning(ed->suffix_entry, "");
             entry_set_warning(ed->unit_name_entry, "");
 
-            valid = valid && ed->unit_location_entry->check_ends_json();
-            valid = valid && ed->symbol_location_entry->check_ends_json();
+            valid = ed->unit_location_entry->check_ends_json() && valid;
+            valid = ed->symbol_location_entry->check_ends_json() && valid;
 
             check_entry_not_empty(ed->unit_name_entry, "Unit name is empty");
             std::string unit_filename = ed->unit_location_entry->get_filename();
@@ -1094,7 +1095,7 @@ void PartWizard::update_steps()
     steps_grid->show_all();
 }
 
-PartWizard *PartWizard::create(const UUID &pkg_uuid, const std::string &bp, class Pool *po,
+PartWizard *PartWizard::create(const UUID &pkg_uuid, const std::string &bp, class Pool &po,
                                class PoolProjectManagerAppWindow *aw)
 {
     PartWizard *w;
@@ -1109,11 +1110,11 @@ PartWizard *PartWizard::create(const UUID &pkg_uuid, const std::string &bp, clas
 PartWizard::~PartWizard()
 {
     for (auto &it : units) {
-        auto filename = pool->get_tmp_filename(ObjectType::UNIT, it.second.uuid);
+        auto filename = pool.get_tmp_filename(ObjectType::UNIT, it.second.uuid);
         Gio::File::create_for_path(filename)->remove();
     }
     for (auto &it : symbols) {
-        auto filename = pool->get_tmp_filename(ObjectType::SYMBOL, it.second);
+        auto filename = pool.get_tmp_filename(ObjectType::SYMBOL, it.second);
         Gio::File::create_for_path(filename)->remove();
     }
 }
