@@ -23,10 +23,7 @@
 #include <thread>
 #include "nlohmann/json.hpp"
 #include "widgets/where_used_box.hpp"
-
-#ifdef G_OS_WIN32
-#undef ERROR
-#endif
+#include "util/win32_undef.hpp"
 
 namespace horizon {
 
@@ -224,6 +221,7 @@ PoolNotebook::PoolNotebook(const std::string &bp, class PoolProjectManagerAppWin
     construct_packages();
     construct_parts();
     construct_frames();
+    construct_decals();
 
     {
         if (PoolManager::get().get_pools().count(pool.get_base_path())) {
@@ -244,9 +242,10 @@ PoolNotebook::PoolNotebook(const std::string &bp, class PoolProjectManagerAppWin
         remote_box->unreference();
 
         signal_switch_page().connect([this](Gtk::Widget *page, int page_num) {
-            if (page == remote_box && !remote_box->prs_refreshed_once) {
-                remote_box->handle_refresh_prs();
-                remote_box->prs_refreshed_once = true;
+            if (in_destruction())
+                return;
+            if (page == remote_box) {
+                remote_box->login_once();
             }
         });
     }
@@ -266,7 +265,7 @@ PoolNotebook::PoolNotebook(const std::string &bp, class PoolProjectManagerAppWin
     }
 
     for (const auto &it_tab : pool_parametric.get_tables()) {
-        auto br = Gtk::manage(new PoolBrowserParametric(&pool, &pool_parametric, it_tab.first));
+        auto br = Gtk::manage(new PoolBrowserParametric(pool, pool_parametric, it_tab.first));
         br->show();
         add_context_menu(br);
         br->signal_activated().connect([this, br] { go_to(ObjectType::PART, br->get_selected()); });
@@ -303,6 +302,7 @@ void PoolNotebook::add_context_menu(PoolBrowser *br)
 {
     ObjectType ty = br->get_type();
     br->add_context_menu_item("Delete", [this, ty](const UUID &uu) { handle_delete(ty, uu); });
+    br->add_context_menu_item("Move/rename", [this, ty](const UUID &uu) { handle_move_rename(ty, uu); });
     br->add_context_menu_item("Copy path", [this, ty](const UUID &uu) { handle_copy_path(ty, uu); });
     br->add_context_menu_item("Copy UUID", [this, ty](const UUID &uu) {
         auto clip = Gtk::Clipboard::get();
@@ -379,6 +379,49 @@ void PoolNotebook::handle_delete(ObjectType ty, const UUID &uu)
     }
 }
 
+
+void PoolNotebook::handle_move_rename(ObjectType ty, const UUID &uu)
+{
+    auto filename = pool.get_filename(ty, uu);
+    auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
+
+    GtkFileChooserNative *native =
+            gtk_file_chooser_native_new("Move item", top->gobj(), GTK_FILE_CHOOSER_ACTION_SAVE, "_Save", "_Cancel");
+    auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
+    chooser->set_current_name(Glib::path_get_basename(filename));
+    while (1) {
+        chooser->set_current_folder(Glib::path_get_dirname(filename));
+        auto resp = gtk_native_dialog_run(GTK_NATIVE_DIALOG(native));
+        if (resp == GTK_RESPONSE_ACCEPT) {
+            const std::string new_filename = chooser->get_filename();
+            try {
+                std::string e;
+                if (!pool.check_filename(ty, new_filename, &e)) {
+                    throw std::runtime_error(e);
+                }
+                Gio::File::create_for_path(filename)->move(Gio::File::create_for_path(new_filename));
+                pool_update();
+                return;
+            }
+            catch (const std::exception &e) {
+                Gtk::MessageDialog md(*top, "Error moving item", false /* use_markup */, Gtk::MESSAGE_ERROR,
+                                      Gtk::BUTTONS_OK);
+                md.set_secondary_text(e.what());
+                md.run();
+            }
+            catch (const Gio::Error &e) {
+                Gtk::MessageDialog md(*top, "Error moving item", false /* use_markup */, Gtk::MESSAGE_ERROR,
+                                      Gtk::BUTTONS_OK);
+                md.set_secondary_text(e.what());
+                md.run();
+            }
+        }
+        else {
+            return;
+        }
+    }
+}
+
 void PoolNotebook::handle_copy_path(ObjectType ty, const UUID &uu)
 {
     auto filename = pool.get_filename(ty, uu);
@@ -388,12 +431,17 @@ void PoolNotebook::handle_copy_path(ObjectType ty, const UUID &uu)
 
 void PoolNotebook::go_to(ObjectType type, const UUID &uu)
 {
-    browsers.at(type)->go_to(uu);
-    static const std::map<ObjectType, int> pages = {
-            {ObjectType::UNIT, 0},    {ObjectType::SYMBOL, 1}, {ObjectType::ENTITY, 2}, {ObjectType::PADSTACK, 3},
-            {ObjectType::PACKAGE, 4}, {ObjectType::PART, 5},   {ObjectType::FRAME, 6},
-    };
-    set_current_page(pages.at(type));
+    auto br = browsers.at(type);
+    br->go_to(uu);
+    Gtk::Widget *last_ancestor = nullptr;
+    Gtk::Widget *ancestor = br;
+    while (ancestor != this) {
+        last_ancestor = ancestor;
+        ancestor = ancestor->get_parent();
+    }
+    auto page = page_num(*last_ancestor);
+    assert(page != -1);
+    set_current_page(page);
 }
 
 const UUID &PoolNotebook::get_pool_uuid() const
@@ -406,7 +454,7 @@ void PoolNotebook::show_duplicate_window(ObjectType ty, const UUID &uu)
     if (!uu)
         return;
     if (!duplicate_window) {
-        duplicate_window = new DuplicateWindow(&pool, ty, uu);
+        duplicate_window = new DuplicateWindow(pool, ty, uu);
         duplicate_window->present();
         duplicate_window->signal_hide().connect([this] {
             auto files_duplicated = duplicate_window->get_filenames();
