@@ -11,56 +11,12 @@
 #include "util/github_client.hpp"
 #include "util/str_util.hpp"
 #include "util/util.hpp"
+#include "github_login_window.hpp"
 #include <iomanip>
+#include "util/win32_undef.hpp"
 
 namespace horizon {
 
-class SignInDialog : public Gtk::Dialog {
-public:
-    SignInDialog(Gtk::Window *parent);
-
-    Gtk::Entry *user_entry = nullptr;
-    Gtk::Entry *password_entry = nullptr;
-};
-
-SignInDialog::SignInDialog(Gtk::Window *parent)
-    : Gtk::Dialog("Sign in with GitHub", *parent,
-                  Gtk::DialogFlags::DIALOG_MODAL | Gtk::DialogFlags::DIALOG_USE_HEADER_BAR)
-{
-    add_button("Cancel", Gtk::ResponseType::RESPONSE_CANCEL);
-    add_button("Sign in", Gtk::ResponseType::RESPONSE_OK);
-    set_default_response(Gtk::RESPONSE_OK);
-    auto grid = Gtk::manage(new Gtk::Grid);
-    grid->set_row_spacing(10);
-    grid->set_column_spacing(10);
-    grid->set_halign(Gtk::ALIGN_CENTER);
-
-    int top = 0;
-    user_entry = Gtk::manage(new Gtk::Entry);
-    grid_attach_label_and_widget(grid, "Username", user_entry, top);
-
-    password_entry = Gtk::manage(new Gtk::Entry);
-    password_entry->set_visibility(false);
-    grid_attach_label_and_widget(grid, "Password", password_entry, top);
-
-    user_entry->signal_activate().connect([this] { password_entry->grab_focus(); });
-    password_entry->signal_activate().connect([this] { response(Gtk::RESPONSE_OK); });
-
-    auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 20));
-    box->property_margin() = 20;
-    box->pack_start(*grid, true, true, 0);
-
-    auto la = Gtk::manage(new Gtk::Label());
-    la->set_markup(
-            "Your credentials will not be saved.\nDon't have a GitHub account? "
-            "<a href='https://github.com/join'>Sign up</a>");
-    la->get_style_context()->add_class("dim-label");
-    box->pack_start(*la, false, false, 0);
-
-    get_content_area()->add(*box);
-    box->show_all();
-}
-
 class ConfirmPrDialog : public Gtk::Dialog {
 public:
     ConfirmPrDialog(Gtk::Window *parent);
@@ -113,6 +69,8 @@ ConfirmPrDialog::ConfirmPrDialog(Gtk::Window *parent)
     box->show_all();
 }
 
+static const std::string github_client_id = "094ac4cbd2e4a51820b4";
+
 PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, PoolNotebook *nb)
     : Gtk::Box(cobject), notebook(nb)
 {
@@ -133,9 +91,57 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
     x->get_widget("remote_pr_title_entry", pr_title_entry);
     x->get_widget("remote_pr_body_textview", pr_body_textview);
     x->get_widget("pr_spinner", pr_spinner);
+    x->get_widget("remote_login_button", login_button);
+    x->get_widget("remote_logout_button", logout_button);
+    x->get_widget("show_only_my_prs_cb", show_only_my_prs_cb);
+    x->get_widget("pr_update_button", pr_update_button);
+    x->get_widget("pr_update_cancel_button", pr_update_cancel_button);
+    logout_button->hide();
+
+    login_button->signal_clicked().connect([this] {
+        if (Glib::file_test(get_token_filename(), Glib::FILE_TEST_IS_REGULAR) && login_succeeded) {
+            update_login();
+            return;
+        }
+
+        auto lw = GitHubLoginWindow::create(get_token_filename(), github_client_id);
+        auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
+        lw->set_transient_for(*top);
+        lw->set_modal(true);
+        lw->signal_hide().connect([this, lw] {
+            delete lw;
+            update_login();
+        });
+        lw->present();
+    });
+
+    logout_button->signal_clicked().connect([this] {
+        auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
+        Gtk::MessageDialog md(*top, "Log out?", false, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_NONE, true);
+        md.set_secondary_text(
+                "Before logging out, you might want to <a "
+                "href=\"https://github.com/settings/connections/applications/"
+                        + github_client_id + "\">revoke</a> the authentication token.",
+                true);
+        md.add_button("Cancel", Gtk::RESPONSE_CANCEL);
+        md.add_button("Log out", Gtk::RESPONSE_OK);
+        md.set_default_response(Gtk::RESPONSE_CANCEL);
+        if (md.run() == Gtk::RESPONSE_OK) {
+            Gio::File::create_for_path(get_token_filename())->remove();
+            gh_username.clear();
+            update_my_prs();
+            gh_token.clear();
+            update_login();
+            upgrade_label->set_text("");
+        }
+    });
+
+    pr_update_cancel_button->signal_clicked().connect([this] { set_pr_update_mode(0, ""); });
 
     pull_requests_listbox->set_header_func(sigc::ptr_fun(header_func_separator));
 
+    show_only_my_prs_cb->signal_toggled().connect(sigc::mem_fun(*this, &PoolRemoteBox::update_prs));
+
     autofree_ptr<git_repository> repo(git_repository_free);
     if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
         throw std::runtime_error("error opening repo");
@@ -162,6 +168,7 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
 
     upgrade_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::handle_remote_upgrade));
     create_pr_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::handle_create_pr));
+    pr_update_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::handle_update_pr));
     refresh_prs_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::handle_refresh_prs));
 
     pr_status_dispatcher.attach(pr_spinner);
@@ -181,11 +188,14 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
         upgrade_spinner->property_active() = !git_thread_error;
         upgrade_label->set_text(git_thread_status);
         if (gh_username.size()) {
-            gh_signed_in_label->set_text(gh_username);
+            gh_signed_in_label->set_text("Logged in: " + gh_username);
         }
         else {
-            gh_signed_in_label->set_text("not signed in");
+            gh_signed_in_label->set_text("Not logged in");
         }
+        login_button->set_visible(gh_username.size() == 0);
+        logout_button->set_visible(gh_username.size() || login_succeeded == false);
+        update_my_prs();
         if (!git_thread_busy) {
             notebook->pool_updating = false;
         }
@@ -196,7 +206,7 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
             if (dia.get_merged())
                 notebook->pool_update();
         }
-        if (git_thread_mode == GitThreadMode::PULL_REQUEST && !git_thread_busy && !git_thread_error) {
+        else if (git_thread_mode == GitThreadMode::PULL_REQUEST && !git_thread_busy && !git_thread_error) {
             items_merge.clear();
             models_merge.clear();
             update_items_merge();
@@ -204,6 +214,13 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
             pr_body_textview->get_buffer()->set_text("");
             handle_refresh_prs();
         }
+        else if (git_thread_mode == GitThreadMode::PULL_REQUEST_UPDATE && !git_thread_busy && !git_thread_error) {
+            set_pr_update_mode(0, "");
+            handle_refresh_prs();
+        }
+        else if (git_thread_mode == GitThreadMode::LOGIN && !git_thread_busy && !git_thread_error) {
+            handle_refresh_prs();
+        }
     });
 
 
@@ -221,6 +238,14 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
         merge_items_view->append_column(*tvc);
     }
 
+    merge_items_view->signal_row_activated().connect(
+            [this](const Gtk::TreeModel::Path &path, Gtk::TreeViewColumn *col) {
+                Gtk::TreeModel::Row row = *item_store->get_iter(path);
+                const ObjectType type = row[list_columns.type];
+                if (type != ObjectType::MODEL_3D)
+                    notebook->go_to(type, row[list_columns.uuid]);
+            });
+
     merge_items_view->append_column("Name", list_columns.name);
 
     merge_items_clear_button->signal_clicked().connect([this] {
@@ -262,49 +287,205 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
 
     pr_body_textview->get_buffer()->signal_changed().connect([this] { update_body_placeholder_label(); });
 
+    set_pr_update_mode(0, "");
+}
 
+int diff_file_cb_c(const git_diff_delta *delta, float progress, void *pl)
+{
+    auto &diff_files = *reinterpret_cast<std::vector<std::string> *>(pl);
+    if (delta->status == GIT_DELTA_ADDED || delta->status == GIT_DELTA_MODIFIED) {
+        diff_files.push_back(delta->new_file.path);
+    }
+    return 0;
+}
+
+static std::pair<git_oid, autofree_ptr<git_tree>> find_base(git_repository *repo, git_commit *latest_commit)
+{
+    git_oid oid_base;
+    autofree_ptr<git_tree> base_tree(git_tree_free);
+    {
+        autofree_ptr<git_object> master_obj(git_object_free);
+        if (git_revparse_single(&master_obj.ptr, repo, "master") != 0) {
+            throw std::runtime_error("error looking up master");
+        }
+        if (git_merge_base(&oid_base, repo, git_object_id(reinterpret_cast<git_object *>(latest_commit)),
+                           git_object_id(master_obj))
+            != 0) {
+            throw std::runtime_error("error finding merge base");
+        }
+
+        autofree_ptr<git_commit> base_commit(git_commit_free);
+        if (git_commit_lookup(&base_commit.ptr, repo, &oid_base) != 0) {
+            throw std::runtime_error("error finding merge base commit");
+        }
+        if (git_commit_tree(&base_tree.ptr, base_commit) != 0) {
+            throw std::runtime_error("error finding merge base tree");
+        }
+    }
+    return {oid_base, std::move(base_tree)};
+}
+
+void PoolRemoteBox::pr_diff_file_cb(const git_diff_delta *delta)
+{
+    SQLite::Query q(notebook->pool.db, "INSERT INTO 'git_files' VALUES (?)");
+    q.bind(1, std::string(delta->new_file.path));
+    q.step();
+}
+
+int PoolRemoteBox::pr_diff_file_cb_c(const git_diff_delta *delta, float progress, void *pl)
+{
+    auto self = reinterpret_cast<PoolRemoteBox *>(pl);
+    self->pr_diff_file_cb(delta);
+    return 0;
+}
+
+
+void PoolRemoteBox::set_pr_update_mode(unsigned int pr, const std::string branch_name)
+{
+    pr_update_button->set_visible(pr);
+    pr_update_cancel_button->set_visible(pr);
+    create_pr_button->set_visible(!pr);
+    pr_update_nr = pr;
+    pr_update_branch = branch_name;
+    pr_update_button->set_label("Update PR #" + std::to_string(pr));
+    update_prs();
+    if (!pr) {
+        items_merge.clear();
+        models_merge.clear();
+        update_items_merge();
+        pr_title_entry->set_text("");
+        pr_body_textview->get_buffer()->set_text("");
+        return;
+    }
+
+    autofree_ptr<git_repository> repo(git_repository_free);
+    if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
+        throw std::runtime_error("error opening repo");
+    }
+
+    autofree_ptr<git_reference> branch(git_reference_free);
+    if (git_branch_lookup(&branch.ptr, repo, branch_name.c_str(), GIT_BRANCH_LOCAL) != 0) {
+        throw std::runtime_error("branch lookup: " + branch_name);
+    }
+
+    autofree_ptr<git_commit> latest_commit(git_commit_free);
+    if (git_reference_peel(reinterpret_cast<git_object **>(&latest_commit.ptr), branch, GIT_OBJ_COMMIT) != 0) {
+        throw std::runtime_error("branch peel");
+    }
+
+    pr_title_entry->set_text(git_commit_message(latest_commit));
+
+
+    auto [oid_base, base_tree] = find_base(repo, latest_commit);
+
+    autofree_ptr<git_tree> tree(git_tree_free);
+    git_commit_tree(&tree.ptr, latest_commit);
+
+    autofree_ptr<git_diff> diff(git_diff_free);
+    git_diff_options diff_opts = GIT_DIFF_OPTIONS_INIT;
+    git_diff_tree_to_tree(&diff.ptr, repo, base_tree, tree, &diff_opts);
+
+
+    notebook->pool.db.execute("CREATE TEMP TABLE 'git_files' ('git_filename' TEXT NOT NULL);");
+    notebook->pool.db.execute("BEGIN TRANSACTION");
+    git_diff_foreach(diff, &PoolRemoteBox::pr_diff_file_cb_c, nullptr, nullptr, nullptr, this);
+    notebook->pool.db.execute("COMMIT");
+
+    SQLite::Query q(
+            notebook->pool.db,
+            "SELECT type, uuid, name, filename FROM git_files LEFT JOIN "
+            "(SELECT type, uuid, name, filename FROM all_items_view UNION ALL SELECT DISTINCT 'model_3d' AS type, "
+            "'00000000-0000-0000-0000-000000000000' as uuid, '' as name, model_filename as filename FROM models) "
+            "ON filename=git_filename WHERE filename IS NOT NULL");
+    while (q.step()) {
+        const ObjectType type = object_type_lut.lookup(q.get<std::string>(0));
+        const UUID uu = q.get<std::string>(1);
+        const auto filename = q.get<std::string>(3);
+        if (type == ObjectType::MODEL_3D) {
+            models_merge.emplace(filename);
+        }
+        else {
+            items_merge.emplace(type, uu);
+        }
+    }
     update_items_merge();
+
+    notebook->pool.db.execute("DROP TABLE 'git_files'");
 }
 
 class PullRequestItemBox : public Gtk::Box {
 public:
-    PullRequestItemBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, const json &j);
-    static PullRequestItemBox *create(const json &j);
-    const std::string filename;
+    PullRequestItemBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, const json &j,
+                       const std::string &gh_user, bool no_update);
+    static PullRequestItemBox *create(const json &j, const std::string &gh_user, bool no_update);
+    const std::string gh_user;
+    const int pr_number;
+    const std::string branch_name;
+
+public:
+    typedef sigc::signal<void> type_signal_update_clicked;
+    type_signal_update_clicked signal_update_clicked()
+    {
+        return s_signal_update_clicked;
+    }
 
 private:
+    type_signal_update_clicked s_signal_update_clicked;
 };
 
-PullRequestItemBox::PullRequestItemBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, const json &j)
-    : Gtk::Box(cobject)
+PullRequestItemBox::PullRequestItemBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, const json &j,
+                                       const std::string &a_gh_user, bool no_update)
+    : Gtk::Box(cobject), gh_user(a_gh_user), pr_number(j.at("number")), branch_name(j.at("head").at("ref"))
 {
     Gtk::Label *name_label, *descr_label, *number_label;
     Gtk::LinkButton *link_button;
+    Gtk::Button *update_button;
     x->get_widget("pull_request_item_name_label", name_label);
     x->get_widget("pull_request_item_descr_label", descr_label);
     x->get_widget("pull_request_item_number_label", number_label);
     x->get_widget("pull_request_item_link", link_button);
+    x->get_widget("pull_request_update_button", update_button);
+
+    update_button->signal_clicked().connect([this] { s_signal_update_clicked.emit(); });
 
     name_label->set_text(j.at("title").get<std::string>());
-    int nr = j.at("number");
-    number_label->set_text("#" + std::to_string(nr));
-    link_button->set_label("Open on GitHub");
+    number_label->set_text("#" + std::to_string(pr_number));
     link_button->set_uri(j.at("_links").at("html").at("href").get<std::string>());
-    std::string open_user = j.at("user").at("login");
-    std::string created_at = j.at("created_at");
+    const std::string open_user = j.at("user").at("login");
+
+    const bool is_pool_mgr_branch = Glib::Regex::match_simple(
+            "^pool_mgr_[0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-"
+            "f]{12}$",
+            branch_name);
+
+    update_button->set_visible(!no_update && open_user == gh_user && is_pool_mgr_branch);
+    const std::string created_at = j.at("created_at");
     descr_label->set_text("opened by " + open_user + " on " + created_at);
 }
 
-PullRequestItemBox *PullRequestItemBox::create(const json &j)
+PullRequestItemBox *PullRequestItemBox::create(const json &j, const std::string &gh_user, bool no_update)
 {
     PullRequestItemBox *w;
     Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create();
     x->add_from_resource("/org/horizon-eda/horizon/pool-prj-mgr/window.ui", "pull_request_item");
-    x->get_widget_derived("pull_request_item", w, j);
+    x->get_widget_derived("pull_request_item", w, j, gh_user, no_update);
     w->reference();
     return w;
 }
 
+void PoolRemoteBox::update_my_prs()
+{
+    if (gh_username.size()) {
+        if (!show_only_my_prs_cb->get_sensitive())
+            show_only_my_prs_cb->set_active(true);
+        show_only_my_prs_cb->set_sensitive(true);
+    }
+    else {
+        show_only_my_prs_cb->set_active(false);
+        show_only_my_prs_cb->set_sensitive(false);
+    }
+}
+
 void PoolRemoteBox::update_prs()
 {
     {
@@ -314,8 +495,15 @@ void PoolRemoteBox::update_prs()
         }
     }
 
-    for (auto it = pull_requests.cbegin(); it != pull_requests.cend(); ++it) {
-        auto box = PullRequestItemBox::create(it.value());
+    for (const auto &it : pull_requests) {
+        if (gh_username.size() && show_only_my_prs_cb->get_active()) {
+            std::string user = it.at("user").at("login");
+            if (user != gh_username)
+                continue;
+        }
+
+        auto box = PullRequestItemBox::create(it, gh_username, pr_update_nr != 0);
+        box->signal_update_clicked().connect([this, box] { set_pr_update_mode(box->pr_number, box->branch_name); });
         pull_requests_listbox->append(*box);
         box->show();
         box->unreference();
@@ -459,16 +647,6 @@ void PoolRemoteBox::handle_create_pr()
         return;
 
     auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
-    if (gh_username.size() == 0) {
-        SignInDialog dia_sign_in(top);
-        if (dia_sign_in.run() == Gtk::RESPONSE_OK) {
-            gh_username = dia_sign_in.user_entry->get_text();
-            gh_password = dia_sign_in.password_entry->get_text();
-        }
-        else {
-            return;
-        }
-    }
 
     { // confirm user name
         autofree_ptr<git_repository> repo(git_repository_free);
@@ -518,6 +696,29 @@ void PoolRemoteBox::handle_create_pr()
     thr.detach();
 }
 
+void PoolRemoteBox::handle_update_pr()
+{
+    if (notebook->pool_updating)
+        return;
+
+    if (!pr_update_nr)
+        return;
+
+    notebook->pool_updating = true;
+    git_thread_busy = true;
+    git_thread_error = false;
+    git_thread_mode = GitThreadMode::PULL_REQUEST_UPDATE;
+
+    pr_body = pr_body_textview->get_buffer()->get_text();
+    pr_title = pr_title_entry->get_text();
+    trim(pr_title);
+    trim(pr_body);
+
+    std::thread thr(&PoolRemoteBox::update_pr_thread, this);
+
+    thr.detach();
+}
+
 void PoolRemoteBox::update_items_merge()
 {
     item_store->clear();
@@ -558,6 +759,7 @@ void PoolRemoteBox::refresh_prs_thread()
         pr_status_dispatcher.set_status(StatusDispatcher::Status::BUSY, "Refreshing");
         GitHubClient client;
         pull_requests = client.get_pull_requests(gh_owner, gh_repo);
+        std::cout << "get prs" << std::endl;
         /*pull_requests = json::array();
         {
                 json j;
@@ -661,7 +863,16 @@ void PoolRemoteBox::remote_upgrade_thread()
             git_thread_status = "Updating local pool...";
         }
         git_thread_dispatcher.emit();
-        pool_update(notebook->base_path);
+        unsigned int n_errors = 0;
+        pool_update(notebook->base_path,
+                    [&n_errors](PoolUpdateStatus status, std::string filename, std::string message) {
+                        if (status == PoolUpdateStatus::FILE_ERROR) {
+                            n_errors++;
+                        }
+                    });
+        if (n_errors) {
+            throw std::runtime_error("local pool update encountered errors");
+        }
 
         {
             std::lock_guard<std::mutex> lock(git_thread_mutex);
@@ -681,6 +892,88 @@ void PoolRemoteBox::remote_upgrade_thread()
     }
 }
 
+std::string PoolRemoteBox::get_token_filename() const
+{
+    return Glib::build_filename(notebook->remote_repo, ".auth.json");
+}
+
+bool PoolRemoteBox::update_login()
+{
+    if (gh_username.size())
+        return false;
+    if (git_thread_busy)
+        return false;
+    const auto auth_avail = Glib::file_test(get_token_filename(), Glib::FILE_TEST_IS_REGULAR);
+    if (!auth_avail) {
+        gh_signed_in_label->set_text("Not logged in");
+        login_button->show();
+        logout_button->hide();
+    }
+    else {
+        if (notebook->pool_updating)
+            return false;
+        notebook->pool_updating = true;
+        git_thread_busy = true;
+        git_thread_error = false;
+        git_thread_mode = GitThreadMode::LOGIN;
+
+        std::thread thr(&PoolRemoteBox::login_thread, this);
+
+        thr.detach();
+        return true;
+    }
+    return false;
+}
+
+void PoolRemoteBox::login_once()
+{
+    if (logged_in_once)
+        return;
+
+    if (!update_login())
+        handle_refresh_prs();
+    logged_in_once = true;
+}
+
+void PoolRemoteBox::login_thread()
+{
+    try {
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            git_thread_status = "Logging in...";
+        }
+        git_thread_dispatcher.emit();
+
+        auto j_auth = load_json_from_file(get_token_filename());
+
+        auto token = j_auth.at("token");
+
+        GitHubClient client;
+        auto user_info = client.login_token(token);
+
+        gh_username = user_info.at("login");
+        gh_token = token;
+
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            git_thread_busy = false;
+            git_thread_status = "Logged in";
+        }
+        login_succeeded = true;
+        git_thread_dispatcher.emit();
+    }
+    catch (const std::exception &e) {
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            login_succeeded = false;
+            git_thread_busy = false;
+            git_thread_error = true;
+            git_thread_status = "Error: " + std::string(e.what());
+        }
+        git_thread_dispatcher.emit();
+    }
+}
+
 void PoolRemoteBox::checkout_master(git_repository *repo)
 {
     autofree_ptr<git_object> treeish(git_object_free);
@@ -697,6 +990,108 @@ void PoolRemoteBox::checkout_master(git_repository *repo)
     }
 }
 
+struct PushPayload {
+    git_cred_userpass_payload userpass;
+    std::string ref_status;
+};
+
+static int push_cred_cb(git_cred **out, const char *url, const char *user_from_url, unsigned int allowed_types,
+                        void *payload)
+{
+    auto pl = reinterpret_cast<PushPayload *>(payload);
+    return git_cred_userpass(out, url, user_from_url, allowed_types, &pl->userpass);
+}
+
+static int push_ref_cb(const char *refname, const char *status, void *data)
+{
+    auto pl = reinterpret_cast<PushPayload *>(data);
+    if (status)
+        pl->ref_status += std::string(refname) + ":" + std::string(status) + " ";
+    return 0;
+}
+
+git_oid PoolRemoteBox::items_to_tree(git_repository *repo)
+{
+    autofree_ptr<git_index> index(git_index_free);
+    if (git_repository_index(&index.ptr, repo) != 0) {
+        throw std::runtime_error("index error");
+    }
+
+    std::set<std::string> files_merge;
+
+    for (const auto &it : items_merge) {
+        SQLite::Query q(notebook->pool.db,
+                        "SELECT filename FROM all_items_view WHERE "
+                        "uuid = ? AND type = ?");
+        q.bind(1, it.second);
+        q.bind(2, object_type_lut.lookup_reverse(it.first));
+        if (q.step()) {
+            std::string filename = q.get<std::string>(0);
+            files_merge.insert(filename);
+        }
+    }
+    files_merge.insert(models_merge.begin(), models_merge.end());
+
+    for (std::string filename : files_merge) {
+        std::cout << "merge " << filename << std::endl;
+        std::string filename_src = Glib::build_filename(notebook->base_path, filename);
+        std::string filename_dest = Glib::build_filename(notebook->remote_repo, filename);
+        std::string dirname_dest = Glib::path_get_dirname(filename_dest);
+        if (!Glib::file_test(dirname_dest, Glib::FILE_TEST_IS_DIR))
+            Gio::File::create_for_path(dirname_dest)->make_directory_with_parents();
+        Gio::File::create_for_path(filename_src)
+                ->copy(Gio::File::create_for_path(filename_dest), Gio::FILE_COPY_OVERWRITE);
+
+#ifdef G_OS_WIN32
+        replace_backslash(filename);
+#endif
+
+        if (git_index_add_bypath(index, filename.c_str()) != 0) {
+            auto last_error = giterr_last();
+            std::string err_str = last_error->message;
+            throw std::runtime_error("add error: " + err_str);
+        }
+    }
+
+    if (git_index_write(index) != 0) {
+        throw std::runtime_error("error saving index");
+    }
+
+    git_oid tree_oid;
+    if (git_index_write_tree(&tree_oid, index) != 0) {
+        throw std::runtime_error("error saving index");
+    }
+
+    return tree_oid;
+}
+
+void PoolRemoteBox::push_branch(git_remote *remote, const std::string &branch_name)
+{
+    git_push_options push_opts = GIT_PUSH_OPTIONS_INIT;
+    push_opts.callbacks.credentials = push_cred_cb;
+    push_opts.callbacks.push_update_reference = push_ref_cb;
+    PushPayload payload;
+    payload.userpass.username = gh_username.c_str();
+    payload.userpass.password = gh_token.c_str();
+    push_opts.callbacks.payload = &payload;
+
+    std::string refspec = "refs/heads/" + branch_name; // + ":refs/heads/" + new_branch_name;
+    char *refspec_c = strdup(refspec.c_str());
+
+    const git_strarray refs = {&refspec_c, 1};
+
+    if (git_remote_push(remote, &refs, &push_opts) != 0) {
+        auto err = giterr_last();
+        std::string errstr = err->message;
+        free(refspec_c);
+        throw std::runtime_error("error pushing " + errstr);
+    }
+    free(refspec_c);
+    if (payload.ref_status.size()) {
+        throw std::runtime_error("error pushing " + payload.ref_status);
+    }
+}
+
 void PoolRemoteBox::create_pr_thread()
 {
     try {
@@ -707,16 +1102,18 @@ void PoolRemoteBox::create_pr_thread()
         }
         git_thread_dispatcher.emit();
 
+        if (gh_token.size() == 0) {
+            throw std::runtime_error("not logged in");
+        }
+
         GitHubClient client;
-        json user_info;
         try {
-            user_info = client.login(gh_username, gh_password);
-            gh_username = user_info.at("login");
+            client.login_token(gh_token);
             git_thread_dispatcher.emit();
         }
         catch (const std::runtime_error &e) {
             gh_username = "";
-            throw std::runtime_error("can't sign in");
+            throw std::runtime_error("can't log in");
         }
 
         if (items_merge.size() == 0 && models_merge.size() == 0) {
@@ -742,25 +1139,6 @@ void PoolRemoteBox::create_pr_thread()
             throw std::runtime_error("error opening repo");
         }
 
-
-        std::string user_email;
-        std::string user_name;
-        std::cout << std::setw(4) << user_info << std::endl;
-        try {
-            user_email = user_info.at("email");
-        }
-        catch (...) {
-            user_email = "horizon@0x83.eu";
-        }
-
-        try {
-            user_name = user_info.at("name");
-        }
-        catch (...) {
-            user_name = "horizon pool manager";
-        }
-
-
         autofree_ptr<git_remote> my_remote(git_remote_free);
         if (git_remote_lookup(&my_remote.ptr, repo, gh_username.c_str()) != 0) {
             // remote doesn't exist, fork upstream
@@ -827,57 +1205,7 @@ void PoolRemoteBox::create_pr_thread()
             }
         }
 
-        git_oid tree_oid;
-        {
-            autofree_ptr<git_index> index(git_index_free);
-            if (git_repository_index(&index.ptr, repo) != 0) {
-                throw std::runtime_error("index error");
-            }
-
-            std::set<std::string> files_merge;
-
-            for (const auto &it : items_merge) {
-                SQLite::Query q(notebook->pool.db,
-                                "SELECT filename FROM all_items_view WHERE "
-                                "uuid = ? AND type = ?");
-                q.bind(1, it.second);
-                q.bind(2, object_type_lut.lookup_reverse(it.first));
-                if (q.step()) {
-                    std::string filename = q.get<std::string>(0);
-                    files_merge.insert(filename);
-                }
-            }
-            files_merge.insert(models_merge.begin(), models_merge.end());
-
-            for (std::string filename : files_merge) {
-                std::cout << "merge " << filename << std::endl;
-                std::string filename_src = Glib::build_filename(notebook->base_path, filename);
-                std::string filename_dest = Glib::build_filename(notebook->remote_repo, filename);
-                std::string dirname_dest = Glib::path_get_dirname(filename_dest);
-                if (!Glib::file_test(dirname_dest, Glib::FILE_TEST_IS_DIR))
-                    Gio::File::create_for_path(dirname_dest)->make_directory_with_parents();
-                Gio::File::create_for_path(filename_src)
-                        ->copy(Gio::File::create_for_path(filename_dest), Gio::FILE_COPY_OVERWRITE);
-
-#ifdef G_OS_WIN32
-                replace_backslash(filename);
-#endif
-
-                if (git_index_add_bypath(index, filename.c_str()) != 0) {
-                    auto last_error = giterr_last();
-                    std::string err_str = last_error->message;
-                    throw std::runtime_error("add error: " + err_str);
-                }
-            }
-
-            if (git_index_write(index) != 0) {
-                throw std::runtime_error("error saving index");
-            }
-
-            if (git_index_write_tree(&tree_oid, index) != 0) {
-                throw std::runtime_error("error saving index");
-            }
-        }
+        git_oid tree_oid = items_to_tree(repo);
 
         {
             autofree_ptr<git_tree> tree(git_tree_free);
@@ -895,7 +1223,7 @@ void PoolRemoteBox::create_pr_thread()
 
 
             autofree_ptr<git_signature> signature(git_signature_free);
-            git_signature_now(&signature.ptr, user_name.c_str(), user_email.c_str());
+            git_signature_default(&signature.ptr, repo);
 
             git_oid new_commit_oid;
             if (git_commit_create(&new_commit_oid, repo, "HEAD", signature, signature, "UTF-8", pr_title.c_str(), tree,
@@ -913,27 +1241,7 @@ void PoolRemoteBox::create_pr_thread()
 
         checkout_master(repo);
 
-        { // push
-            git_push_options push_opts = GIT_PUSH_OPTIONS_INIT;
-            push_opts.callbacks.credentials = git_cred_userpass;
-            git_cred_userpass_payload payload;
-            payload.username = gh_username.c_str();
-            payload.password = gh_password.c_str();
-            push_opts.callbacks.payload = &payload;
-
-            std::string refspec = "refs/heads/" + new_branch_name;
-            char *refspec_c = strdup(refspec.c_str());
-
-            const git_strarray refs = {&refspec_c, 1};
-
-            if (git_remote_push(my_remote, &refs, &push_opts) != 0) {
-                auto err = giterr_last();
-                std::string errstr = err->message;
-                free(refspec_c);
-                throw std::runtime_error("error pushing " + errstr);
-            }
-            free(refspec_c);
-        }
+        push_branch(my_remote, new_branch_name);
 
         {
             std::lock_guard<std::mutex> lock(git_thread_mutex);
@@ -970,4 +1278,158 @@ void PoolRemoteBox::create_pr_thread()
         git_thread_dispatcher.emit();
     }
 }
+
+void PoolRemoteBox::update_pr_thread()
+{
+    try {
+
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            git_thread_status = "Authenticating...";
+        }
+        git_thread_dispatcher.emit();
+
+        if (gh_token.size() == 0) {
+            throw std::runtime_error("not logged in");
+        }
+
+        GitHubClient client;
+        try {
+            client.login_token(gh_token);
+            git_thread_dispatcher.emit();
+        }
+        catch (const std::runtime_error &e) {
+            gh_username = "";
+            throw std::runtime_error("can't log in");
+        }
+
+        if (items_merge.size() == 0 && models_merge.size() == 0) {
+            throw std::runtime_error("no items to merge");
+        }
+
+        if (pr_title.size() == 0) {
+            throw std::runtime_error("please enter title");
+        }
+
+
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            git_thread_status = "Opening repository...";
+        }
+        git_thread_dispatcher.emit();
+
+        autofree_ptr<git_repository> repo(git_repository_free);
+        if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
+            throw std::runtime_error("error opening repo");
+        }
+
+        autofree_ptr<git_remote> my_remote(git_remote_free);
+        if (git_remote_lookup(&my_remote.ptr, repo, gh_username.c_str()) != 0) {
+            throw std::runtime_error("can't find remote");
+        }
+
+
+        autofree_ptr<git_reference> branch(git_reference_free);
+        if (git_branch_lookup(&branch.ptr, repo, pr_update_branch.c_str(), GIT_BRANCH_LOCAL) != 0) {
+            throw std::runtime_error("branch lookup");
+        }
+
+        autofree_ptr<git_commit> latest_commit(git_commit_free);
+        if (git_reference_peel(reinterpret_cast<git_object **>(&latest_commit.ptr), branch, GIT_OBJ_COMMIT) != 0) {
+            throw std::runtime_error("branch peel");
+        }
+
+        auto [oid_base, base_tree] = find_base(repo, latest_commit);
+
+        {
+            git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
+            checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE;
+            if (git_checkout_tree(repo, reinterpret_cast<git_object *>(base_tree.ptr), &checkout_opts) != 0) {
+                throw std::runtime_error("error checking out tree");
+            }
+        }
+        if (git_repository_set_head_detached(repo, &oid_base) != 0) {
+            throw std::runtime_error("error setting detached head");
+        }
+
+        {
+            auto tree_oid = items_to_tree(repo);
+            autofree_ptr<git_tree> tree(git_tree_free);
+            if (git_tree_lookup(&tree.ptr, repo, &tree_oid) != 0) {
+                throw std::runtime_error("error looking up tree");
+            }
+
+            autofree_ptr<git_signature> signature(git_signature_free);
+            git_signature_default(&signature.ptr, repo);
+
+            const git_commit *parent_commit_p = latest_commit.ptr;
+
+            git_oid new_commit_oid;
+            if (git_commit_create(&new_commit_oid, repo, NULL, signature, signature, "UTF-8", pr_title.c_str(), tree, 1,
+                                  &parent_commit_p)
+                != 0) {
+                throw std::runtime_error("error committing");
+            }
+
+            git_reference *new_ref = nullptr;
+            if (git_reference_set_target(&new_ref, branch, &new_commit_oid, "update pr") != 0) {
+                throw std::runtime_error("error updating branch");
+            }
+        }
+        checkout_master(repo);
+        {
+            autofree_ptr<git_object> obj(git_object_free);
+            git_revparse_single(&obj.ptr, repo, "master");
+
+            git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
+            if (git_reset(repo, obj, GIT_RESET_HARD, &checkout_opts) != 0) {
+                throw std::runtime_error("error resetting");
+            }
+        }
+
+
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            git_thread_status = "Pushing...";
+        }
+        git_thread_dispatcher.emit();
+
+        push_branch(my_remote, pr_update_branch);
+
+        if (pr_body.size()) {
+            {
+                std::lock_guard<std::mutex> lock(git_thread_mutex);
+                git_thread_status = "Adding comment...";
+            }
+            git_thread_dispatcher.emit();
+            client.add_issue_comment(gh_owner, gh_repo, pr_update_nr, pr_body);
+        }
+
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            git_thread_busy = false;
+            git_thread_status = "Done";
+        }
+        git_thread_dispatcher.emit();
+    }
+    catch (const std::exception &e) {
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            git_thread_busy = false;
+            git_thread_error = true;
+            git_thread_status = "Error: " + std::string(e.what());
+        }
+        git_thread_dispatcher.emit();
+    }
+    catch (const Gio::Error &e) {
+        {
+            std::lock_guard<std::mutex> lock(git_thread_mutex);
+            git_thread_busy = false;
+            git_thread_error = true;
+            git_thread_status = "IO Error: " + std::string(e.what());
+        }
+        git_thread_dispatcher.emit();
+    }
+}
+
 } // namespace horizon
