@@ -25,6 +25,12 @@
 #include "autosave_recovery_dialog.hpp"
 #include "util/item_set.hpp"
 #include "prj-mgr/pool_cache_monitor.hpp"
+#include "pool/pool_cached.hpp"
+
+#ifdef G_OS_WIN32
+#include <winsock2.h>
+#endif
+#include "util/win32_undef.hpp"
 
 namespace horizon {
 PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject,
@@ -68,6 +74,8 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
     builder->get_widget("hamburger_menu_button", hamburger_menu_button);
     builder->get_widget("info_bar_pool_not_added", info_bar_pool_not_added);
     builder->get_widget("info_bar_pool_doc", info_bar_pool_doc);
+    builder->get_widget("info_bar_version", info_bar_version);
+    builder->get_widget("version_label", version_label);
 
     set_view_mode(ViewMode::OPEN);
 
@@ -442,7 +450,9 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
         }
     }
     else if (op == "preferences") {
-        app->show_preferences_window(timestamp);
+        auto win = app->show_preferences_window(timestamp);
+        if (j.count("page"))
+            win->show_page(j.at("page"));
     }
     else if (op == "show-in-pool-mgr") {
         UUID uu = j.at("uuid").get<std::string>();
@@ -583,6 +593,9 @@ void PoolProjectManagerAppWindow::handle_cancel()
 
 void PoolProjectManagerAppWindow::save_project()
 {
+    if (project_read_only)
+        return;
+    set_version_info("");
     save_json_to_file(project_filename, project->serialize());
     project_needs_save = false;
 }
@@ -848,6 +861,7 @@ void PoolProjectManagerAppWindow::set_view_mode(ViewMode mode)
         hamburger_menu_button->show();
         header->set_title("Horizon EDA");
         update_recent_items();
+        set_version_info("");
         break;
 
     case ViewMode::POOL:
@@ -1094,6 +1108,23 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
         std::string pool_path = prj_pool->base_path;
         check_schema_update(pool_path);
 
+        {
+            project_read_only = false;
+            const auto &version = project->version;
+            const auto version_msg = version.get_message(ObjectType::PROJECT);
+            if (version_msg.size()) {
+                set_version_info(
+                        version_msg
+                        + " This only applies to the project file. Board and Schematic might have different versions.");
+            }
+            else {
+                set_version_info("");
+            }
+            if (version.get_app() < version.get_file()) {
+                project_read_only = true;
+            }
+        }
+
         view_project.label_project_directory->set_text(Glib::path_get_dirname(project_filename));
 
         view_project.pool_info_bar->hide();
@@ -1541,7 +1572,7 @@ void PoolProjectManagerAppWindow::cleanup_pool_cache()
         }
     }
 
-    PoolCacheCleanupDialog dia(pool_cache_window, files_to_delete, models_to_delete, &pool_cached);
+    PoolCacheCleanupDialog dia(pool_cache_window, files_to_delete, models_to_delete, pool_cached);
     dia.run();
 }
 
@@ -1552,4 +1583,16 @@ void PoolProjectManagerAppWindow::update_pool_cache_status_now()
     }
 }
 
+
+void PoolProjectManagerAppWindow::set_version_info(const std::string &s)
+{
+    if (s.size()) {
+        info_bar_show(info_bar_version);
+        version_label->set_markup(s);
+    }
+    else {
+        info_bar_hide(info_bar_version);
+    }
+}
+
 } // namespace horizon
