@@ -6,26 +6,29 @@
 #include <fstream>
 #include <map>
 #include <vector>
+#include "util/file_version.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
 
-class Entity : public UUIDProvider {
+class Entity {
 private:
-    Entity(const UUID &uu, const json &, class Pool &pool);
+    Entity(const UUID &uu, const json &, class IPool &pool);
 
 public:
     Entity(const UUID &uu);
 
-    static Entity new_from_file(const std::string &filename, Pool &pool);
+    static Entity new_from_file(const std::string &filename, IPool &pool);
     UUID uuid;
     std::string name;
     std::string manufacturer;
     std::string prefix;
     std::set<std::string> tags;
     std::map<UUID, Gate> gates;
+
+    FileVersion version;
     json serialize() const;
-    void update_refs(Pool &pool);
+    void update_refs(IPool &pool);
     UUID get_uuid() const;
 };
 } // namespace horizon
