@@ -1,4 +1,4 @@
-#include "pool.hpp"
+#include "ipool.hpp"
 #include "package.hpp"
 #include "util/util.hpp"
 #include "nlohmann/json.hpp"
@@ -57,43 +57,50 @@ json Package::Model::serialize() const
     return j;
 }
 
-Package::Package(const UUID &uu, const json &j, Pool &pool)
+static const unsigned int app_version = 0;
+
+Package::Package(const UUID &uu, const json &j, IPool &pool)
     : uuid(uu), name(j.at("name").get<std::string>()), manufacturer(j.value("manufacturer", "")),
-      parameter_program(this, j.value("parameter_program", ""))
+      parameter_program(this, j.value("parameter_program", "")), version(app_version, j)
 {
+    check_object_type(j, ObjectType::PACKAGE);
+    version.check(ObjectType::PACKAGE, name, uuid);
     {
         const json &o = j.at("junctions");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            junctions.emplace(std::make_pair(u, Junction(u, it.value())));
+            junctions.emplace(std::piecewise_construct, std::forward_as_tuple(u), std::forward_as_tuple(u, it.value()));
         }
     }
     {
         const json &o = j.at("lines");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            lines.emplace(std::make_pair(u, Line(u, it.value(), *this)));
+            lines.emplace(std::piecewise_construct, std::forward_as_tuple(u),
+                          std::forward_as_tuple(u, it.value(), *this));
         }
     }
     {
         const json &o = j.at("arcs");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            arcs.emplace(std::make_pair(u, Arc(u, it.value(), *this)));
+            arcs.emplace(std::piecewise_construct, std::forward_as_tuple(u),
+                         std::forward_as_tuple(u, it.value(), *this));
         }
     }
     {
         const json &o = j.at("texts");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            texts.emplace(std::make_pair(u, Text(u, it.value())));
+            texts.emplace(std::piecewise_construct, std::forward_as_tuple(u), std::forward_as_tuple(u, it.value()));
         }
     }
     {
         const json &o = j.at("pads");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            pads.emplace(std::make_pair(u, Pad(u, it.value(), pool)));
+            pads.emplace(std::piecewise_construct, std::forward_as_tuple(u),
+                         std::forward_as_tuple(u, it.value(), pool));
         }
     }
 
@@ -101,7 +108,7 @@ Package::Package(const UUID &uu, const json &j, Pool &pool)
         const json &o = j["polygons"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            polygons.emplace(std::make_pair(u, Polygon(u, it.value())));
+            polygons.emplace(std::piecewise_construct, std::forward_as_tuple(u), std::forward_as_tuple(u, it.value()));
         }
     }
     if (j.count("keepouts")) {
@@ -158,15 +165,18 @@ Package::Package(const UUID &uu, const json &j, Pool &pool)
         }
         default_model = j.at("default_model").get<std::string>();
     }
+    if (j.count("rules")) {
+        rules.load_from_json(j.at("rules"));
+    }
 }
 
-Package Package::new_from_file(const std::string &filename, Pool &pool)
+Package Package::new_from_file(const std::string &filename, IPool &pool)
 {
     auto j = load_json_from_file(filename);
     return Package(UUID(j.at("uuid").get<std::string>()), j, pool);
 }
 
-Package::Package(const UUID &uu) : uuid(uu), parameter_program(this, "")
+Package::Package(const UUID &uu) : uuid(uu), parameter_program(this, ""), version(app_version)
 {
 }
 
@@ -185,7 +195,7 @@ Package::Package(const Package &pkg)
       lines(pkg.lines), arcs(pkg.arcs), texts(pkg.texts), pads(pkg.pads), polygons(pkg.polygons),
       keepouts(pkg.keepouts), dimensions(pkg.dimensions), pictures(pkg.pictures), parameter_set(pkg.parameter_set),
       parameter_program(pkg.parameter_program), models(pkg.models), default_model(pkg.default_model),
-      alternate_for(pkg.alternate_for), warnings(pkg.warnings)
+      alternate_for(pkg.alternate_for), version(pkg.version), warnings(pkg.warnings)
 {
     update_refs();
 }
@@ -210,11 +220,12 @@ void Package::operator=(Package const &pkg)
     models = pkg.models;
     default_model = pkg.default_model;
     alternate_for = pkg.alternate_for;
+    version = pkg.version;
     warnings = pkg.warnings;
     update_refs();
 }
 
-void Package::update_refs(Pool &pool)
+void Package::update_refs(IPool &pool)
 {
     for (auto &it : pads) {
         it.second.pool_padstack = pool.get_padstack(it.second.pool_padstack.uuid);
@@ -294,9 +305,7 @@ void Package::expand()
         it.second.from->connection_count++;
         it.second.to->connection_count++;
         for (auto &ju : {it.second.from, it.second.to}) {
-            if (ju->layer == 10000) { // none assigned
-                ju->layer = it.second.layer;
-            }
+            ju->layer.merge(it.second.layer);
         }
     }
     for (const auto &it : arcs) {
@@ -304,9 +313,7 @@ void Package::expand()
         it.second.to->connection_count++;
         it.second.center->connection_count++;
         for (auto &ju : {it.second.from, it.second.to}) {
-            if (ju->layer == 10000) { // none assigned
-                ju->layer = it.second.layer;
-            }
+            ju->layer.merge(it.second.layer);
         }
     }
 
@@ -382,6 +389,7 @@ const std::map<int, Layer> &Package::get_layers() const
 json Package::serialize() const
 {
     json j;
+    version.serialize(j);
     j["uuid"] = (std::string)uuid;
     j["type"] = "package";
     j["name"] = name;
@@ -434,6 +442,7 @@ json Package::serialize() const
             j["pictures"][(std::string)it.first] = it.second.serialize();
         }
     }
+    j["rules"] = rules.serialize();
     return j;
 }
 
