@@ -33,6 +33,7 @@ void Pool::clear()
     packages.clear();
     parts.clear();
     frames.clear();
+    decals.clear();
     pool_uuid_cache.clear();
 }
 
@@ -68,6 +69,10 @@ std::string Pool::get_filename(ObjectType type, const UUID &uu, UUID *pool_uuid_
         query = "SELECT filename, pool_uuid FROM frames WHERE uuid = ?";
         break;
 
+    case ObjectType::DECAL:
+        query = "SELECT filename, pool_uuid FROM decals WHERE uuid = ?";
+        break;
+
     default:
         return "";
     }
@@ -105,24 +110,13 @@ std::string Pool::get_filename(ObjectType type, const UUID &uu, UUID *pool_uuid_
 
     switch (type) {
     case ObjectType::UNIT:
-        return Glib::build_filename(bp, filename);
-
     case ObjectType::ENTITY:
-        return Glib::build_filename(bp, filename);
-
     case ObjectType::SYMBOL:
-        return Glib::build_filename(bp, filename);
-
     case ObjectType::PACKAGE:
-        return Glib::build_filename(bp, filename);
-
     case ObjectType::PADSTACK:
-        return Glib::build_filename(bp, filename);
-
     case ObjectType::PART:
-        return Glib::build_filename(bp, filename);
-
     case ObjectType::FRAME:
+    case ObjectType::DECAL:
         return Glib::build_filename(bp, filename);
 
     default:
@@ -137,7 +131,7 @@ const std::string &Pool::get_base_path() const
 
 int Pool::get_required_schema_version()
 { // keep in sync with schema definition
-    return 15;
+    return 16;
 }
 
 std::string Pool::get_tmp_filename(ObjectType type, const UUID &uu) const
@@ -175,6 +169,9 @@ std::string Pool::get_flat_filename(ObjectType type, const UUID &uu) const
     case ObjectType::FRAME:
         return "frame_" + suffix;
 
+    case ObjectType::DECAL:
+        return "decal_" + suffix;
+
     default:
         return "";
     }
@@ -279,7 +276,7 @@ const Part *Pool::get_part(const UUID &uu, UUID *pool_uuid_out)
 
 const Frame *Pool::get_frame(const UUID &uu, UUID *pool_uuid_out)
 {
-    if (parts.count(uu) == 0) {
+    if (frames.count(uu) == 0) {
         std::string path = get_filename(ObjectType::FRAME, uu, pool_uuid_out);
         Frame f = Frame::new_from_file(path);
         frames.insert(std::make_pair(uu, f));
@@ -290,6 +287,19 @@ const Frame *Pool::get_frame(const UUID &uu, UUID *pool_uuid_out)
     return &frames.at(uu);
 }
 
+const Decal *Pool::get_decal(const UUID &uu, UUID *pool_uuid_out)
+{
+    if (decals.count(uu) == 0) {
+        std::string path = get_filename(ObjectType::DECAL, uu, pool_uuid_out);
+        Decal d = Decal::new_from_file(path);
+        decals.insert(std::make_pair(uu, d));
+    }
+    else {
+        get_pool_uuid(ObjectType::DECAL, uu, pool_uuid_out);
+    }
+    return &decals.at(uu);
+}
+
 std::set<UUID> Pool::get_alternate_packages(const UUID &uu)
 {
     std::set<UUID> r;
@@ -314,4 +324,74 @@ std::string Pool::get_model_filename(const UUID &pkg_uuid, const UUID &model_uui
         return "";
     }
 }
+
+static std::string get_dir_for_type(ObjectType type)
+{
+    using O = ObjectType;
+    switch (type) {
+    case O::UNIT:
+        return "units";
+
+    case O::SYMBOL:
+        return "symbols";
+
+    case O::ENTITY:
+        return "entities";
+
+    case O::PADSTACK:
+        return "padstacks";
+
+    case O::PACKAGE:
+        return "packages";
+
+    case O::PART:
+        return "parts";
+
+    case O::FRAME:
+        return "frames";
+
+    case O::DECAL:
+        return "decals";
+
+    default:
+        return "";
+    }
+}
+
+static bool check_type(ObjectType type)
+{
+    return get_dir_for_type(type).size();
+}
+
+bool Pool::check_filename(ObjectType type, const std::string &filename, std::string *error_msg) const
+{
+    if (!check_type(type)) {
+        if (error_msg)
+            *error_msg = "unsupported object type";
+        return false;
+    }
+    if (type == ObjectType::PADSTACK) {
+        const std::string bp_ps = Glib::build_filename(base_path, get_dir_for_type(type));
+        if (Gio::File::create_for_path(filename)->has_prefix(Gio::File::create_for_path(bp_ps)))
+            return true;
+        const std::string bp_pkg = Glib::build_filename(base_path, get_dir_for_type(type));
+        if (Gio::File::create_for_path(filename)->has_prefix(Gio::File::create_for_path(bp_pkg))) {
+            return true;
+        }
+        if (error_msg)
+            *error_msg = "incorrect directory";
+        return false;
+    }
+    else {
+        const std::string bp = Glib::build_filename(base_path, get_dir_for_type(type));
+        auto r = Gio::File::create_for_path(filename)->has_prefix(Gio::File::create_for_path(bp));
+        if (!r) {
+            if (error_msg)
+                *error_msg = "incorrect directory";
+        }
+        return r;
+    }
+}
+
+
 } // namespace horizon
