@@ -2,7 +2,7 @@
 #include "common/junction.hpp"
 #include "common/line.hpp"
 #include "common/lut.hpp"
-#include "pool.hpp"
+#include "ipool.hpp"
 #include "util/util.hpp"
 #include "nlohmann/json.hpp"
 #include <algorithm>
@@ -128,10 +128,14 @@ UUID SymbolPin::get_uuid() const
     return uuid;
 }
 
-Symbol::Symbol(const UUID &uu, const json &j, Pool &pool)
+static const unsigned int app_version = 0;
+
+Symbol::Symbol(const UUID &uu, const json &j, IPool &pool)
     : uuid(uu), unit(pool.get_unit(j.at("unit").get<std::string>())), name(j.value("name", "")),
-      can_expand(j.value("can_expand", false))
+      can_expand(j.value("can_expand", false)), version(app_version, j)
 {
+    check_object_type(j, ObjectType::SYMBOL);
+    version.check(ObjectType::SYMBOL, name, uuid);
     if (j.count("junctions")) {
         const json &o = j["junctions"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
@@ -194,11 +198,11 @@ Symbol::Symbol(const UUID &uu, const json &j, Pool &pool)
     }
 }
 
-Symbol::Symbol(const UUID &uu) : uuid(uu)
+Symbol::Symbol(const UUID &uu) : uuid(uu), version(app_version)
 {
 }
 
-Symbol Symbol::new_from_file(const std::string &filename, Pool &pool)
+Symbol Symbol::new_from_file(const std::string &filename, IPool &pool)
 {
     auto j = load_json_from_file(filename);
     return Symbol(UUID(j.at("uuid").get<std::string>()), j, pool);
@@ -208,10 +212,6 @@ Junction *Symbol::get_junction(const UUID &uu)
 {
     return junctions.count(uu) ? &junctions.at(uu) : nullptr;
 }
-SymbolPin *Symbol::get_symbol_pin(const UUID &uu)
-{
-    return pins.count(uu) ? &pins.at(uu) : nullptr;
-}
 
 void Symbol::update_refs()
 {
@@ -231,7 +231,7 @@ void Symbol::update_refs()
 Symbol::Symbol(const Symbol &sym)
     : uuid(sym.uuid), unit(sym.unit), name(sym.name), pins(sym.pins), junctions(sym.junctions), lines(sym.lines),
       arcs(sym.arcs), texts(sym.texts), polygons(sym.polygons), can_expand(sym.can_expand),
-      text_placements(sym.text_placements)
+      text_placements(sym.text_placements), version(sym.version)
 {
     update_refs();
 }
@@ -249,7 +249,7 @@ void Symbol::operator=(Symbol const &sym)
     polygons = sym.polygons;
     can_expand = sym.can_expand;
     text_placements = sym.text_placements;
-
+    version = sym.version;
     update_refs();
 }
 
@@ -321,6 +321,7 @@ void Symbol::apply_placement(const Placement &p)
 json Symbol::serialize() const
 {
     json j;
+    version.serialize(j);
     j["type"] = "symbol";
     j["name"] = name;
     j["uuid"] = (std::string)uuid;
