@@ -10,16 +10,13 @@
 #include "nlohmann/json_fwd.hpp"
 #include "unit.hpp"
 #include "util/uuid.hpp"
-#include "util/uuid_provider.hpp"
-#include <fstream>
-#include <map>
-#include <set>
-#include <vector>
+#include "symbol/symbol_rules.hpp"
+#include "util/file_version.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
 
-class SymbolPin : public UUIDProvider {
+class SymbolPin {
 public:
     enum class ConnectorStyle { BOX, NONE, NC };
 
@@ -68,17 +65,16 @@ public:
     Pin::Direction direction = Pin::Direction::BIDIRECTIONAL;
 
     json serialize() const;
-    virtual UUID get_uuid() const;
+    UUID get_uuid() const;
 };
 
 class Symbol : public ObjectProvider, public LayerProvider {
 public:
-    Symbol(const UUID &uu, const json &j, class Pool &pool);
+    Symbol(const UUID &uu, const json &j, class IPool &pool);
     Symbol(const UUID &uu);
-    static Symbol new_from_file(const std::string &filename, Pool &pool);
+    static Symbol new_from_file(const std::string &filename, IPool &pool);
     std::pair<Coordi, Coordi> get_bbox(bool all = false) const;
-    virtual Junction *get_junction(const UUID &uu);
-    virtual SymbolPin *get_symbol_pin(const UUID &uu);
+    virtual Junction *get_junction(const UUID &uu) override;
 
     json serialize() const;
 
@@ -104,6 +100,9 @@ public:
     std::map<std::tuple<int, bool, UUID>, Placement> text_placements;
     void apply_placement(const Placement &p);
 
+    SymbolRules rules;
+    FileVersion version;
+
 private:
     void update_refs();
 };
