@@ -1,12 +1,12 @@
 #pragma once
 #include "nlohmann/json_fwd.hpp"
 #include "util/uuid.hpp"
-#include "util/uuid_provider.hpp"
 #include "common/lut.hpp"
 #include <fstream>
 #include <map>
 #include <set>
 #include <vector>
+#include "util/file_version.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
@@ -15,7 +15,7 @@ using json = nlohmann::json;
  * A Pin represents a logical pin of a Unit.
  */
 
-class Pin : public UUIDProvider {
+class Pin {
 public:
     enum class Direction {
         INPUT,
@@ -57,7 +57,7 @@ public:
  * An example for a Unit may be a "single-ended NAND gate".
  * \ref Unit "Units" are stored in an Entity.
  */
-class Unit : public UUIDProvider {
+class Unit {
 private:
     Unit(const UUID &uu, const json &j);
 
@@ -68,6 +68,8 @@ public:
     std::string name;
     std::string manufacturer;
     std::map<UUID, Pin> pins;
+    FileVersion version;
+
     json serialize() const;
     UUID get_uuid() const;
 };
