@@ -19,6 +19,7 @@ std::string rules_check_error_level_to_string(RulesCheckErrorLevel lev);
 class RulesCheckError {
 public:
     RulesCheckError(RulesCheckErrorLevel lev);
+    RulesCheckError(RulesCheckErrorLevel lev, const std::string &comment);
 
     RulesCheckErrorLevel level = RulesCheckErrorLevel::NOT_RUN;
     UUID sheet;
@@ -53,6 +54,10 @@ public:
 
     virtual const Rule *get_rule(RuleID id) const = 0;
     Rule *get_rule(RuleID id);
+    Rule *get_rule_nc(RuleID id)
+    {
+        return get_rule(id);
+    }
 
     virtual const Rule *get_rule(RuleID id, const UUID &uu) const = 0;
     Rule *get_rule(RuleID id, const UUID &uu);
